# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import math

from rally import exceptions


def mean(values):
    """Find the simple average of a list of values.

    :parameter values: non-empty list of numbers

    :returns: float value
    """
    if not values:
        raise exceptions.InvalidArgumentsException(
                                        "the list should be non-empty")
    return math.fsum(values) / len(values)


def percentile(values, percent):
    """Find the percentile of a list of values.

    :parameter values: list of numbers
    :parameter percent: float value from 0.0 to 1.0

    :returns: the percentile of values
    """
    if not values:
        return None
    values.sort()
    k = (len(values) - 1) * percent
    f = math.floor(k)
    c = math.ceil(k)
    if f == c:
        return values[int(k)]
    d0 = values[int(f)] * (c - k)
    d1 = values[int(c)] * (k - f)
    return (d0 + d1)


def get_atomic_actions_data(raw_data):
    """Retrieve detailed (by atomic actions & total runtime) benchmark data.

    :parameter raw_data: list of raw records (scenario runner output)

    :returns: dictionary containing atomic action + total duration lists
              for all atomic action keys
    """
    atomic_actions = []
    for row in raw_data:
        # find first non-error result to get atomic actions names
        if not row["error"] and "atomic_actions" in row:
            atomic_actions = row["atomic_actions"].keys()
    actions_data = {}
    for atomic_action in atomic_actions:
        actions_data[atomic_action] = [
            r["atomic_actions"][atomic_action]
            for r in raw_data
            if r["atomic_actions"].get(atomic_action) is not None]
    actions_data["total"] = [r["duration"] for r in raw_data if not r["error"]]
    return actions_data


def compress(data, limit=1000, merge=None, normalize=None):
    """Enumerate and reduce list of values.

    :param data: data list
    :param limit: int, max length of result list
    :param merge: function that merges two values
    :param normalize: function that guarantees sanity of value
    :returns: items list [(idx1, value1), (idx2, value2) ...]
    """

    if not normalize:
        normalize = lambda i: i and round(float(i), 2) or 0.0

    if not merge:
        merge = lambda a, b: normalize((a + normalize(b)) / 2)

    if len(data) <= limit:
        return [(idx, normalize(v)) for idx, v in enumerate(data, start=1)]

    # For determining which rows should be merged we are using `factor'
    # e.g. if we have 100 rows and should reduce it to 75 then we have
    # factor of 0.25 and delete (merge with previous) each 4th row.
    factor = float(limit) / len(data)
    store = 0.0

    result = []
    first = True

    for idx, value in enumerate(data, start=1):
        store += factor

        if first:
            cur_value = normalize(value)
            first = False
        else:
            cur_value = merge(cur_value, value)

        if store > 1:
            store -= 1
            first = True
            result.append((idx, cur_value))

    if not first:
        result.append((len(data), cur_value))

    return result



# Copyright 2014: The Rally team
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

""" Rally command: show """

from __future__ import print_function

from rally.cmd import cliutils
from rally.cmd import envutils
from rally.common.i18n import _
from rally.common import utils
from rally import db
from rally import exceptions
from rally import objects
from rally import osclients


class ShowCommands(object):
    """Show resources.

    Set of commands that allow you to view resourses, provided by OpenStack
    cloud represented by deployment.
    """

    def _get_endpoints(self, deployment):
        deployment = db.deployment_get(deployment)
        admin = deployment.get("admin")
        admin = [admin] if admin else []

        return admin + deployment.get("users", [])

    @cliutils.deprecated_args(
        "--deploy-id", dest="deployment", type=str,
        required=False, help="UUID of the deployment.")
    @cliutils.args("--deployment", dest="deployment", type=str,
                   required=False, help="UUID or name of a deployment")
    @envutils.with_default_deployment(cli_arg_name="deployment")
    def images(self, deployment=None):
        """Display available images.

        :param deployment: UUID or name of a deployment
        """

        headers = ["UUID", "Name", "Size (B)"]
        mixed_case_fields = ["UUID", "Name"]
        float_cols = ["Size (B)"]
        table_rows = []
        formatters = dict(zip(float_cols,
                              [cliutils.pretty_float_formatter(col)
                               for col in float_cols]))

        try:
            for endpoint_dict in self._get_endpoints(deployment):
                clients = osclients.Clients(objects.Endpoint(**endpoint_dict))
                glance_client = clients.glance()
                for image in glance_client.images.list():
                    data = [image.id, image.name, image.size]
                    table_rows.append(utils.Struct(**dict(zip(headers, data))))

                cliutils.print_list(table_rows,
                                    fields=headers,
                                    formatters=formatters,
                                    mixed_case_fields=mixed_case_fields)

        except exceptions.InvalidArgumentsException as e:
            print(_("Authentication Issues: %s") % e)
            return(1)

    @cliutils.deprecated_args(
        "--deploy-id", dest="deployment", type=str,
        required=False, help="UUID of the deployment.")
    @cliutils.args("--deployment", dest="deployment", type=str,
                   required=False, help="UUID or name of a deployment")
    @envutils.with_default_deployment(cli_arg_name="deployment")
    def flavors(self, deployment=None):
        """Display available flavors.

        :param deployment: UUID or name of a deployment
        """

        headers = ["ID", "Name", "vCPUs", "RAM (MB)", "Swap (MB)", "Disk (GB)"]
        mixed_case_fields = ["ID", "Name", "vCPUs"]
        float_cols = ["RAM (MB)", "Swap (MB)", "Disk (GB)"]
        formatters = dict(zip(float_cols,
                              [cliutils.pretty_float_formatter(col)
                               for col in float_cols]))
        table_rows = []
        try:
            for endpoint_dict in self._get_endpoints(deployment):
                clients = osclients.Clients(objects.Endpoint(**endpoint_dict))
                nova_client = clients.nova()
                for flavor in nova_client.flavors.list():
                    data = [flavor.id, flavor.name, flavor.vcpus,
                            flavor.ram, flavor.swap, flavor.disk]
                    table_rows.append(utils.Struct(**dict(zip(headers, data))))

                cliutils.print_list(table_rows,
                                    fields=headers,
                                    formatters=formatters,
                                    mixed_case_fields=mixed_case_fields)

        except exceptions.InvalidArgumentsException as e:
            print(_("Authentication Issues: %s") % e)
            return(1)

    @cliutils.deprecated_args(
        "--deploy-id", dest="deployment", type=str,
        required=False, help="UUID of the deployment.")
    @cliutils.args("--deployment", dest="deployment", type=str,
                   required=False, help="UUID or name of a deployment")
    @envutils.with_default_deployment(cli_arg_name="deployment")
    def networks(self, deployment=None):
        """Display configured networks."""

        headers = ["ID", "Label", "CIDR"]
        mixed_case_fields = ["ID", "Label", "CIDR"]
        table_rows = []
        try:
            for endpoint_dict in self._get_endpoints(deployment):
                clients = osclients.Clients(objects.Endpoint(**endpoint_dict))
                nova_client = clients.nova()
                for network in nova_client.networks.list():
                    data = [network.id, network.label, network.cidr]
                    table_rows.append(utils.Struct(**dict(zip(headers, data))))

                cliutils.print_list(table_rows,
                                    fields=headers,
                                    mixed_case_fields=mixed_case_fields)
        except exceptions.InvalidArgumentsException as e:
            print(_("Authentication Issues: %s") % e)
            return(1)

    @cliutils.deprecated_args(
        "--deploy-id", dest="deployment", type=str,
        required=False, help="UUID of the deployment.")
    @cliutils.args("--deployment", dest="deployment", type=str,
                   required=False, help="UUID or name of a deployment")
    @envutils.with_default_deployment(cli_arg_name="deployment")
    def secgroups(self, deployment=None):
        """Display security groups."""

        headers = ["ID", "Name", "Description"]
        mixed_case_fields = ["ID", "Name", "Description"]
        table_rows = []
        try:
            for endpoint_dict in self._get_endpoints(deployment):
                clients = osclients.Clients(objects.Endpoint(**endpoint_dict))
                nova_client = clients.nova()
                for secgroup in nova_client.security_groups.list():
                    data = [secgroup.id, secgroup.name,
                            secgroup.description]
                    table_rows.append(utils.Struct(**dict(zip(headers,
                                                              data))))
                cliutils.print_list(
                    table_rows,
                    fields=headers,
                    mixed_case_fields=mixed_case_fields)

        except exceptions.InvalidArgumentsException as e:
            print(_("Authentication Issues: %s") % e)
            return(1)

    @cliutils.deprecated_args(
        "--deploy-id", dest="deployment", type=str,
        required=False, help="UUID of the deployment.")
    @cliutils.args("--deployment", dest="deployment", type=str,
                   required=False, help="UUID or name of a deployment")
    @envutils.with_default_deployment(cli_arg_name="deployment")
    def keypairs(self, deployment=None):
        """Display available ssh keypairs."""

        headers = ["Name", "Fingerprint"]
        mixed_case_fields = ["Name", "Fingerprint"]
        table_rows = []
        try:
            for endpoint_dict in self._get_endpoints(deployment):
                clients = osclients.Clients(objects.Endpoint(**endpoint_dict))
                nova_client = clients.nova()
                for keypair in nova_client.keypairs.list():
                    data = [keypair.name, keypair.fingerprint]
                    table_rows.append(utils.Struct(**dict(zip(headers, data))))
                cliutils.print_list(table_rows,
                                    fields=headers,
                                    mixed_case_fields=mixed_case_fields)

        except exceptions.InvalidArgumentsException as e:
            print(_("Authentication Issues: %s") % e)
            return(1)



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutronclient.common import exceptions as neutron_exceptions
from saharaclient.api import base as saharaclient_base

from rally.benchmark.context.cleanup import base
from rally.benchmark.scenarios.keystone import utils as kutils
from rally.benchmark.wrappers import keystone as keystone_wrapper
from rally.common import log as logging


LOG = logging.getLogger(__name__)


def get_order(start):
    return iter(range(start, start + 99))


class SynchronizedDeletion(object):

    def is_deleted(self):
        return True


class QuotaMixin(SynchronizedDeletion):

    def id(self):
        return self.raw_resource

    def delete(self):
        self._manager().delete(self.raw_resource)

    def list(self):
        return [self.tenant_uuid] if self.tenant_uuid else []


# HEAT

@base.resource("heat", "stacks", order=100)
class HeatStack(base.ResourceManager):
    pass


# NOVA

_nova_order = get_order(200)


@base.resource("nova", "servers", order=next(_nova_order))
class NovaServer(base.ResourceManager):
    pass


@base.resource("nova", "keypairs", order=next(_nova_order))
class NovaKeypair(SynchronizedDeletion, base.ResourceManager):
    pass


@base.resource("nova", "security_groups", order=next(_nova_order))
class NovaSecurityGroup(SynchronizedDeletion, base.ResourceManager):

    def list(self):
        return filter(lambda x: x.name != "default",
                      super(NovaSecurityGroup, self).list())


@base.resource("nova", "quotas", order=next(_nova_order),
               admin_required=True, tenant_resource=True)
class NovaQuotas(QuotaMixin, base.ResourceManager):
    pass


# NEUTRON

_neutron_order = get_order(300)


@base.resource(service=None, resource=None, admin_required=True)
class NeutronMixin(SynchronizedDeletion, base.ResourceManager):
    # Neutron has the best client ever, so we need to override everything

    def _manager(self):
        client = self._admin_required and self.admin or self.user
        return getattr(client, self._service)()

    def id(self):
        return self.raw_resource["id"]

    def delete(self):
        delete_method = getattr(self._manager(), "delete_%s" % self._resource)
        delete_method(self.id())

    def list(self):
        resources = self._resource + "s"
        list_method = getattr(self._manager(), "list_%s" % resources)

        return filter(lambda r: r["tenant_id"] == self.tenant_uuid,
                      list_method({"tenant_id": self.tenant_uuid})[resources])


@base.resource("neutron", "port", order=next(_neutron_order),
               tenant_resource=True)
class NeutronPort(NeutronMixin):

    def delete(self):
        if self.raw_resource["device_owner"] == "network:router_interface":
            self._manager().remove_interface_router(
                self.raw_resource["device_id"],
                {"port_id": self.raw_resource["id"]})
        else:
            try:
                self._manager().delete_port(self.id())
            except neutron_exceptions.PortNotFoundClient:
                # Port can be already auto-deleted, skip silently
                LOG.debug("Port %s was not deleted. Skip silently because "
                          "port can be already auto-deleted."
                          % self.id())


@base.resource("neutron", "router", order=next(_neutron_order),
               tenant_resource=True)
class NeutronRouter(NeutronMixin):
    pass


@base.resource("neutron", "subnet", order=next(_neutron_order),
               tenant_resource=True)
class NeutronSubnet(NeutronMixin):
    pass


@base.resource("neutron", "network", order=next(_neutron_order),
               tenant_resource=True)
class NeutronNetwork(NeutronMixin):
    pass


@base.resource("neutron", "quota", order=next(_neutron_order),
               admin_required=True, tenant_resource=True)
class NeutronQuota(QuotaMixin, NeutronMixin):

    def delete(self):
        self._manager().delete_quota(self.tenant_uuid)


# CINDER

_cinder_order = get_order(400)


@base.resource("cinder", "backups", order=next(_cinder_order),
               tenant_resource=True)
class CinderVolumeBackup(base.ResourceManager):
    pass


@base.resource("cinder", "volume_snapshots", order=next(_cinder_order),
               tenant_resource=True)
class CinderVolumeSnapshot(base.ResourceManager):
    pass


@base.resource("cinder", "transfers", order=next(_cinder_order),
               tenant_resource=True)
class CinderVolumeTransfer(base.ResourceManager):
    pass


@base.resource("cinder", "volumes", order=next(_cinder_order),
               tenant_resource=True)
class CinderVolume(base.ResourceManager):
    pass


@base.resource("cinder", "quotas", order=next(_cinder_order),
               admin_required=True, tenant_resource=True)
class CinderQuotas(QuotaMixin, base.ResourceManager):
    pass


# GLANCE

@base.resource("glance", "images", order=500, tenant_resource=True)
class GlanceImage(base.ResourceManager):

    def list(self):
        return self._manager().list(owner=self.tenant_uuid)


# SAHARA

_sahara_order = get_order(600)


@base.resource("sahara", "job_executions", order=next(_sahara_order),
               tenant_resource=True)
class SaharaJobExecution(SynchronizedDeletion, base.ResourceManager):
    pass


@base.resource("sahara", "jobs", order=next(_sahara_order),
               tenant_resource=True)
class SaharaJob(SynchronizedDeletion, base.ResourceManager):
    pass


@base.resource("sahara", "job_binary_internals", order=next(_sahara_order),
               tenant_resource=True)
class SaharaJobBinaryInternals(SynchronizedDeletion, base.ResourceManager):
    pass


@base.resource("sahara", "job_binaries", order=next(_sahara_order),
               tenant_resource=True)
class SaharaJobBinary(SynchronizedDeletion, base.ResourceManager):
    pass


@base.resource("sahara", "data_sources", order=next(_sahara_order),
               tenant_resource=True)
class SaharaDataSource(SynchronizedDeletion, base.ResourceManager):
    pass


@base.resource("sahara", "clusters", order=next(_sahara_order),
               tenant_resource=True)
class SaharaCluster(base.ResourceManager):

    # Need special treatment for Sahara Cluster because of the way the
    # exceptions are described in:
    # https://github.com/openstack/python-saharaclient/blob/master/
    # saharaclient/api/base.py#L145

    def is_deleted(self):
        try:
            self._manager().get(self.id())
            return False
        except saharaclient_base.APIException as e:
            return e.error_code == 404


@base.resource("sahara", "cluster_templates", order=next(_sahara_order),
               tenant_resource=True)
class SaharaClusterTemplate(SynchronizedDeletion, base.ResourceManager):
    pass


@base.resource("sahara", "node_group_templates", order=next(_sahara_order),
               tenant_resource=True)
class SaharaNodeGroup(SynchronizedDeletion, base.ResourceManager):
    pass


# CEILOMETER

@base.resource("ceilometer", "alarms", order=700, tenant_resource=True)
class CeilometerAlarms(SynchronizedDeletion, base.ResourceManager):

    def id(self):
        return self.raw_resource.alarm_id

    def list(self):
        query = [{
            "field": "project_id",
            "op": "eq",
            "value": self.tenant_uuid
        }]
        return self._manager().list(q=query)


# ZAQAR

@base.resource("zaqar", "queues", order=800)
class ZaqarQueues(SynchronizedDeletion, base.ResourceManager):

    def list(self):
        return self.user.zaqar().queues()


# DESIGNATE

_designate_order = get_order(900)


@base.resource("designate", "domains", order=next(_designate_order))
class Designate(SynchronizedDeletion, base.ResourceManager):
    pass


@base.resource("designate", "servers", order=next(_designate_order),
               admin_required=True, perform_for_admin_only=True)
class DesignateServer(SynchronizedDeletion, base.ResourceManager):
    pass


# MISTRAL

@base.resource("mistral", "workbooks", order=1100, tenant_resource=True)
class MistralWorkbooks(SynchronizedDeletion, base.ResourceManager):
    def delete(self):
        self._manager().delete(self.raw_resource.name)


# KEYSTONE

_keystone_order = get_order(9000)


class KeystoneMixin(SynchronizedDeletion):

    def _manager(self):
        return keystone_wrapper.wrap(getattr(self.admin, self._service)())

    def delete(self):
        delete_method = getattr(self._manager(), "delete_%s" % self._resource)
        delete_method(self.id())

    def list(self):
        # TODO(boris-42): We should use such stuff in all list commands.
        resources = self._resource + "s"
        list_method = getattr(self._manager(), "list_%s" % resources)

        return filter(kutils.is_temporary, list_method())


@base.resource("keystone", "user", order=next(_keystone_order),
               admin_required=True, perform_for_admin_only=True)
class KeystoneUser(KeystoneMixin, base.ResourceManager):
    pass


@base.resource("keystone", "project", order=next(_keystone_order),
               admin_required=True, perform_for_admin_only=True)
class KeystoneProject(KeystoneMixin, base.ResourceManager):
    pass


@base.resource("keystone", "service", order=next(_keystone_order),
               admin_required=True, perform_for_admin_only=True)
class KeystoneService(KeystoneMixin, base.ResourceManager):
    pass


@base.resource("keystone", "role", order=next(_keystone_order),
               admin_required=True, perform_for_admin_only=True)
class KeystoneRole(KeystoneMixin, base.ResourceManager):
    pass



# Copyright 2014: Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from rally.benchmark.scenarios.neutron import network
from tests.unit import test

NEUTRON_NETWORKS = "rally.benchmark.scenarios.neutron.network.NeutronNetworks"


class NeutronNetworksTestCase(test.TestCase):

    @mock.patch(NEUTRON_NETWORKS + "._list_networks")
    @mock.patch(NEUTRON_NETWORKS + "._create_network")
    def test_create_and_list_networks(self, mock_create, mock_list):
        neutron_scenario = network.NeutronNetworks()

        # Default options
        network_create_args = {}
        neutron_scenario.create_and_list_networks(
            network_create_args=network_create_args)
        mock_create.assert_called_once_with(network_create_args)
        mock_list.assert_called_once_with()

        mock_create.reset_mock()
        mock_list.reset_mock()

        # Explicit network name is specified
        network_create_args = {"name": "given-name"}
        neutron_scenario.create_and_list_networks(
            network_create_args=network_create_args)
        mock_create.assert_called_once_with(network_create_args)
        mock_list.assert_called_once_with()

    @mock.patch(NEUTRON_NETWORKS + "._update_network")
    @mock.patch(NEUTRON_NETWORKS + "._create_network", return_value={
         "network": {
             "id": "network-id",
             "name": "network-name",
             "admin_state_up": False
         }
    })
    def test_create_and_update_networks(self,
                                        mock_create_network,
                                        mock_update_network):
        scenario = network.NeutronNetworks()

        network_update_args = {"name": "_updated", "admin_state_up": True}

        # Default options
        scenario.create_and_update_networks(
            network_update_args=network_update_args)

        mock_create_network.assert_called_once_with({})

        mock_update_network.assert_has_calls(
            [mock.call(mock_create_network.return_value, network_update_args)])

        mock_create_network.reset_mock()
        mock_update_network.reset_mock()

        # Explicit network name is specified
        network_create_args = {"name": "network-name", "admin_state_up": False}

        scenario.create_and_update_networks(
            network_create_args=network_create_args,
            network_update_args=network_update_args)
        mock_create_network.assert_called_once_with(network_create_args)
        mock_update_network.assert_has_calls(
            [mock.call(mock_create_network.return_value, network_update_args)])

    @mock.patch(NEUTRON_NETWORKS + "._delete_network")
    @mock.patch(NEUTRON_NETWORKS + "._create_network")
    def test_create_and_delete_networks(self, mock_create, mock_delete):
        neutron_scenario = network.NeutronNetworks()

        # Default options
        network_create_args = {}
        neutron_scenario.create_and_delete_networks()
        mock_create.assert_called_once_with(network_create_args)
        self.assertEqual(1, mock_delete.call_count)

        mock_create.reset_mock()
        mock_delete.reset_mock()

        # Explict network name is specified
        network_create_args = {"name": "given-name"}
        neutron_scenario.create_and_delete_networks(
                                    network_create_args=network_create_args)
        mock_create.assert_called_once_with(network_create_args)
        self.assertEqual(1, mock_delete.call_count)

    @mock.patch(NEUTRON_NETWORKS + "._list_subnets")
    @mock.patch(NEUTRON_NETWORKS + "._create_network_and_subnets")
    def test_create_and_list_subnets(self,
                                     mock_create_network_and_subnets,
                                     mock_list):
        scenario = network.NeutronNetworks()
        subnets_per_network = 4
        subnet_cidr_start = "default_cidr"

        mock_create_network_and_subnets.reset_mock()
        mock_list.reset_mock()

        # Default options
        scenario.create_and_list_subnets(
            subnets_per_network=subnets_per_network,
            subnet_cidr_start=subnet_cidr_start)

        mock_create_network_and_subnets.assert_has_calls(
            [mock.call({}, {}, subnets_per_network,
                       subnet_cidr_start)])
        mock_list.assert_called_once_with()

        mock_create_network_and_subnets.reset_mock()
        mock_list.reset_mock()

        # Custom options
        scenario.create_and_list_subnets(
            subnet_create_args={"allocation_pools": []},
            subnet_cidr_start="custom_cidr",
            subnets_per_network=subnets_per_network)

        mock_create_network_and_subnets.assert_has_calls(
            [mock.call({}, {"allocation_pools": []},
                       subnets_per_network, "custom_cidr")])
        mock_list.assert_called_once_with()

    @mock.patch(NEUTRON_NETWORKS + "._update_subnet")
    @mock.patch(NEUTRON_NETWORKS + "._create_network_and_subnets")
    def test_create_and_update_subnets(self,
                                       mock_create_network_and_subnets,
                                       mock_update_subnet):
        scenario = network.NeutronNetworks()
        subnets_per_network = 1
        subnet_cidr_start = "default_cidr"
        net = {
            "network": {
                "id": "network-id"
            }
        }
        subnet = {
            "subnet": {
                "name": "subnet-name",
                "id": "subnet-id",
                "enable_dhcp": False
            }
        }
        mock_create_network_and_subnets.return_value = (net, [subnet])
        subnet_update_args = {"name": "_updated", "enable_dhcp": True}

        mock_create_network_and_subnets.reset_mock()
        mock_update_subnet.reset_mock()

        # Default options
        scenario.create_and_update_subnets(
            subnet_update_args=subnet_update_args,
            subnet_cidr_start=subnet_cidr_start,
            subnets_per_network=subnets_per_network)

        mock_create_network_and_subnets.assert_has_calls(
            [mock.call({}, {}, subnets_per_network, subnet_cidr_start)])
        mock_update_subnet.assert_has_calls(
            [mock.call(subnet, subnet_update_args)])

        mock_create_network_and_subnets.reset_mock()
        mock_update_subnet.reset_mock()

        # Custom options
        subnet_cidr_start = "custom_cidr"
        scenario.create_and_update_subnets(
            subnet_update_args=subnet_update_args,
            subnet_create_args={"allocation_pools": []},
            subnet_cidr_start=subnet_cidr_start,
            subnets_per_network=subnets_per_network)

        mock_create_network_and_subnets.assert_has_calls(
            [mock.call({}, {"allocation_pools": []}, subnets_per_network,
                       subnet_cidr_start)])
        mock_update_subnet.assert_has_calls(
            [mock.call(subnet, subnet_update_args)])

    @mock.patch(NEUTRON_NETWORKS + "._delete_subnet")
    @mock.patch(NEUTRON_NETWORKS + "._create_network_and_subnets")
    def test_create_and_delete_subnets(self,
                                       mock_create_network_and_subnets,
                                       mock_delete):
        scenario = network.NeutronNetworks()
        net = {
            "network": {
                "id": "network-id"
            }
        }
        subnet = {
            "subnet": {
                "name": "subnet-name",
                "id": "subnet-id",
                "enable_dhcp": False
            }
        }
        mock_create_network_and_subnets.return_value = (net, [subnet])
        subnets_per_network = 1
        subnet_cidr_start = "default_cidr"

        mock_create_network_and_subnets.reset_mock()
        mock_delete.reset_mock()

        # Default options
        scenario.create_and_delete_subnets(
            subnets_per_network=subnets_per_network,
            subnet_cidr_start=subnet_cidr_start)

        mock_create_network_and_subnets.assert_has_calls(
            [mock.call({}, {}, subnets_per_network,
                       subnet_cidr_start)])

        mock_delete.assert_has_calls([mock.call(subnet)])

        mock_create_network_and_subnets.reset_mock()
        mock_delete.reset_mock()

        # Custom options
        subnet_cidr_start = "custom_cidr"
        scenario.create_and_delete_subnets(
            subnet_create_args={"allocation_pools": []},
            subnet_cidr_start="custom_cidr",
            subnets_per_network=subnets_per_network)

        mock_create_network_and_subnets.assert_has_calls(
            [mock.call({}, {"allocation_pools": []}, subnets_per_network,
                       subnet_cidr_start)])
        mock_delete.assert_has_calls([mock.call(subnet)])

    @mock.patch(NEUTRON_NETWORKS + "._list_routers")
    @mock.patch(NEUTRON_NETWORKS + "._create_router")
    @mock.patch(NEUTRON_NETWORKS + "._create_network_and_subnets")
    @mock.patch(NEUTRON_NETWORKS + ".clients")
    def test_create_and_list_routers(self,
                                     mock_clients,
                                     mock_create_network_and_subnets,
                                     mock_create_router,
                                     mock_list):
        scenario = network.NeutronNetworks()
        subnets_per_network = 1
        subnet_cidr_start = "default_cidr"

        net = {
            "network": {
                "id": "network-id"
            }
        }
        subnet = {
            "subnet": {
                "name": "subnet-name",
                "id": "subnet-id",
                "enable_dhcp": False
            }
        }
        mock_create_network_and_subnets.return_value = (net, [subnet])
        mock_clients("neutron").add_interface_router = mock.Mock()
        router = {
            "router": {
                "name": "router-name",
                "id": "router-id"
            }
        }
        mock_create_router.return_value = router

        # Default options
        scenario.create_and_list_routers(
            subnet_cidr_start=subnet_cidr_start,
            subnets_per_network=subnets_per_network)
        mock_create_network_and_subnets.assert_has_calls(
            [mock.call({}, {}, subnets_per_network, subnet_cidr_start)])

        mock_create_router.assert_has_calls(
            [mock.call({})] * subnets_per_network)

        mock_clients("neutron").add_interface_router.assert_has_calls(
            [mock.call(router["router"]["id"],
                       {"subnet_id": subnet["subnet"]["id"]})
             ] * subnets_per_network)

        mock_create_network_and_subnets.reset_mock()
        mock_create_router.reset_mock()

        mock_clients("neutron").add_interface_router.reset_mock()
        mock_list.reset_mock()

        # Custom options
        subnet_cidr_start = "custom_cidr"
        subnet_create_args = {"allocation_pools": []}
        router_create_args = {"admin_state_up": False}
        scenario.create_and_list_routers(
            subnet_create_args=subnet_create_args,
            subnet_cidr_start="custom_cidr",
            subnets_per_network=subnets_per_network,
            router_create_args=router_create_args)

        mock_create_network_and_subnets.assert_has_calls(
            [mock.call({}, subnet_create_args, subnets_per_network,
             subnet_cidr_start)])

        mock_create_router.assert_has_calls(
            [mock.call(router_create_args)] * subnets_per_network)
        mock_clients("neutron").add_interface_router.assert_has_calls(
            [mock.call(router["router"]["id"],
                       {"subnet_id": subnet["subnet"]["id"]})
             ] * subnets_per_network)

        mock_list.assert_called_once_with()

    @mock.patch(NEUTRON_NETWORKS + "._update_router")
    @mock.patch(NEUTRON_NETWORKS + "._create_router")
    @mock.patch(NEUTRON_NETWORKS + "._create_network_and_subnets")
    @mock.patch(NEUTRON_NETWORKS + ".clients")
    def test_create_and_update_routers(self,
                                       mock_clients,
                                       mock_create_network_and_subnets,
                                       mock_create_router,
                                       mock_update_router):
        scenario = network.NeutronNetworks()
        subnets_per_network = 1
        subnet_cidr_start = "default_cidr"

        net = {
            "network": {
                "id": "network-id"
            }
        }
        subnet = {
            "subnet": {
                "name": "subnet-name",
                "id": "subnet-id",
                "enable_dhcp": False
            }
        }
        router = {
            "router": {
                "name": "router-name",
                "id": "router-id"
            }
        }
        router_update_args = {
            "name": "_updated",
            "admin_state_up": False
        }
        mock_create_router.return_value = router
        mock_create_network_and_subnets.return_value = (net, [subnet])
        mock_clients("neutron").add_interface_router = mock.Mock()

        # Default options
        scenario.create_and_update_routers(
            router_update_args=router_update_args,
            subnet_cidr_start=subnet_cidr_start,
            subnets_per_network=subnets_per_network)

        mock_create_network_and_subnets.assert_has_calls(
            [mock.call({}, {}, subnets_per_network, subnet_cidr_start)])

        mock_create_router.assert_has_calls(
            [mock.call({})] * subnets_per_network)
        mock_clients("neutron").add_interface_router.assert_has_calls(
            [mock.call(router["router"]["id"],
                       {"subnet_id": subnet["subnet"]["id"]})
             ] * subnets_per_network)

        mock_update_router.assert_has_calls(
            [mock.call(router, router_update_args)
             ] * subnets_per_network)

        mock_create_network_and_subnets.reset_mock()
        mock_create_router.reset_mock()
        mock_clients("neutron").add_interface_router.reset_mock()
        mock_update_router.reset_mock()

        # Custom options
        subnet_cidr_start = "custom_cidr"
        subnet_create_args = {"allocation_pools": []}
        router_create_args = {"admin_state_up": False}
        scenario.create_and_update_routers(
            router_update_args=router_update_args,
            subnet_create_args=subnet_create_args,
            subnet_cidr_start="custom_cidr",
            subnets_per_network=subnets_per_network,
            router_create_args=router_create_args)

        mock_create_network_and_subnets.assert_has_calls(
            [mock.call({}, subnet_create_args, subnets_per_network,
             subnet_cidr_start)])

        mock_create_router.assert_has_calls(
            [mock.call(router_create_args)] * subnets_per_network)
        mock_clients("neutron").add_interface_router.assert_has_calls(
            [mock.call(router["router"]["id"],
                       {"subnet_id": subnet["subnet"]["id"]})
             ] * subnets_per_network)

        mock_update_router.assert_has_calls(
            [mock.call(router, router_update_args)
             ] * subnets_per_network)

    @mock.patch(NEUTRON_NETWORKS + "._delete_router")
    @mock.patch(NEUTRON_NETWORKS + "._create_router")
    @mock.patch(NEUTRON_NETWORKS + "._create_network_and_subnets")
    @mock.patch(NEUTRON_NETWORKS + ".clients")
    def test_create_and_delete_routers(self,
                                       mock_clients,
                                       mock_create_network_and_subnets,
                                       mock_create_router,
                                       mock_delete_router):
        scenario = network.NeutronNetworks()
        subnets_per_network = 1
        subnet_cidr_start = "default_cidr"

        net = {
            "network": {
                "id": "network-id"
            }
        }
        subnet = {
            "subnet": {
                "name": "subnet-name",
                "id": "subnet-id",
                "enable_dhcp": False
            }
        }
        router = {
            "router": {
                "name": "router-name",
                "id": "router-id"
            }
        }

        mock_create_router.return_value = router
        mock_create_network_and_subnets.return_value = (net, [subnet])
        mock_clients("neutron").add_interface_router = mock.Mock()

        # Default options
        scenario.create_and_delete_routers(
            subnet_cidr_start=subnet_cidr_start,
            subnets_per_network=subnets_per_network)

        mock_create_network_and_subnets.assert_has_calls(
            [mock.call({}, {}, subnets_per_network, subnet_cidr_start)])

        mock_create_router.assert_has_calls(
            [mock.call({})] * subnets_per_network)
        mock_clients("neutron").add_interface_router.assert_has_calls(
            [mock.call(router["router"]["id"],
                       {"subnet_id": subnet["subnet"]["id"]})
             ] * subnets_per_network)

        mock_delete_router.assert_has_calls(
            [mock.call(router)] * subnets_per_network)

        mock_create_network_and_subnets.reset_mock()
        mock_create_router.reset_mock()
        mock_clients("neutron").add_interface_router.reset_mock()
        mock_delete_router.reset_mock()

        # Custom options
        subnet_cidr_start = "custom_cidr"
        subnet_create_args = {"allocation_pools": []}
        router_create_args = {"admin_state_up": False}
        scenario.create_and_delete_routers(
            subnet_create_args=subnet_create_args,
            subnet_cidr_start="custom_cidr",
            subnets_per_network=subnets_per_network,
            router_create_args=router_create_args)

        mock_create_network_and_subnets.assert_has_calls(
            [mock.call({}, subnet_create_args, subnets_per_network,
             subnet_cidr_start)])

        mock_create_router.assert_has_calls(
            [mock.call(router_create_args)] * subnets_per_network)
        mock_clients("neutron").add_interface_router.assert_has_calls(
            [mock.call(router["router"]["id"],
                       {"subnet_id": subnet["subnet"]["id"]})
             ] * subnets_per_network)

        mock_delete_router.assert_has_calls(
            [mock.call(router)] * subnets_per_network)

    @mock.patch(NEUTRON_NETWORKS + "._generate_random_name")
    @mock.patch(NEUTRON_NETWORKS + "._list_ports")
    @mock.patch(NEUTRON_NETWORKS + "._create_port")
    @mock.patch(NEUTRON_NETWORKS + "._create_network")
    def test_create_and_list_ports(self,
                                   mock_create_network,
                                   mock_create_port,
                                   mock_list,
                                   mock_random_name):
        scenario = network.NeutronNetworks()
        mock_random_name.return_value = "random-name"
        net = {"network": {"id": "fake-id"}}
        mock_create_network.return_value = net
        ports_per_network = 10

        self.assertRaises(TypeError, scenario.create_and_list_ports)

        mock_create_network.reset_mock()

        # Defaults
        scenario.create_and_list_ports(ports_per_network=ports_per_network)
        mock_create_network.assert_called_once_with({})
        self.assertEqual(mock_create_port.mock_calls,
                         [mock.call(net, {})] * ports_per_network)
        mock_list.assert_called_once_with()

        mock_create_network.reset_mock()
        mock_create_port.reset_mock()
        mock_list.reset_mock()

        # Custom options
        scenario.create_and_list_ports(
            network_create_args={"name": "given-name"},
            port_create_args={"allocation_pools": []},
            ports_per_network=ports_per_network)
        mock_create_network.assert_called_once_with({"name": "given-name"})
        self.assertEqual(
            mock_create_port.mock_calls,
            [mock.call(net, {"allocation_pools": []})] * ports_per_network)
        mock_list.assert_called_once_with()

    @mock.patch(NEUTRON_NETWORKS + "._generate_random_name")
    @mock.patch(NEUTRON_NETWORKS + "._update_port")
    @mock.patch(NEUTRON_NETWORKS + "._create_port", return_value={
         "port": {
             "name": "port-name",
             "id": "port-id",
             "admin_state_up": True
         }
    })
    @mock.patch(NEUTRON_NETWORKS + "._create_network", return_value={
         "network": {
             "id": "fake-id"
         }
    })
    def test_create_and_update_ports(self,
                                     mock_create_network,
                                     mock_create_port,
                                     mock_update_port,
                                     mock_random_name):
        scenario = network.NeutronNetworks()
        mock_random_name.return_value = "random-name"
        ports_per_network = 10

        port_update_args = {
            "name": "_updated",
            "admin_state_up": False
        }

        # Defaults
        scenario.create_and_update_ports(
            port_update_args=port_update_args,
            ports_per_network=ports_per_network)
        mock_create_network.assert_called_once_with({})

        mock_create_port.assert_has_calls(
            [mock.call({"network": {"id": "fake-id"}},
                       {})] * ports_per_network)
        mock_update_port.assert_has_calls(
            [mock.call(mock_create_port.return_value, port_update_args)
             ] * ports_per_network)

        mock_create_network.reset_mock()
        mock_create_port.reset_mock()
        mock_update_port.reset_mock()

        # Custom options
        scenario.create_and_update_ports(
            port_update_args=port_update_args,
            network_create_args={"name": "given-name"},
            port_create_args={"allocation_pools": []},
            ports_per_network=ports_per_network)
        mock_create_network.assert_called_once_with({"name": "given-name"})
        mock_create_port.assert_has_calls(
            [mock.call({"network": {"id": "fake-id"}},
                       {"allocation_pools": []})] * ports_per_network)
        mock_update_port.assert_has_calls(
            [mock.call(mock_create_port.return_value, port_update_args)
             ] * ports_per_network)

    @mock.patch(NEUTRON_NETWORKS + "._generate_random_name")
    @mock.patch(NEUTRON_NETWORKS + "._delete_port")
    @mock.patch(NEUTRON_NETWORKS + "._create_port")
    @mock.patch(NEUTRON_NETWORKS + "._create_network")
    def test_create_and_delete_ports(self,
                                     mock_create_network,
                                     mock_create_port,
                                     mock_delete,
                                     mock_random_name):
        scenario = network.NeutronNetworks()
        mock_random_name.return_value = "random-name"
        net = {"network": {"id": "fake-id"}}
        mock_create_network.return_value = net
        ports_per_network = 10

        self.assertRaises(TypeError, scenario.create_and_delete_ports)

        mock_create_network.reset_mock()

        # Default options
        scenario.create_and_delete_ports(ports_per_network=ports_per_network)
        mock_create_network.assert_called_once_with({})
        self.assertEqual(mock_create_port.mock_calls,
                         [mock.call(net, {})] * ports_per_network)
        self.assertEqual(mock_delete.mock_calls,
                         [mock.call(mock_create_port())] * ports_per_network)

        mock_create_network.reset_mock()
        mock_create_port.reset_mock()
        mock_delete.reset_mock()

        # Custom options
        scenario.create_and_delete_ports(
            network_create_args={"name": "given-name"},
            port_create_args={"allocation_pools": []},
            ports_per_network=ports_per_network)
        mock_create_network.assert_called_once_with({"name": "given-name"})
        self.assertEqual(
            mock_create_port.mock_calls,
            [mock.call(net, {"allocation_pools": []})] * ports_per_network)
        self.assertEqual(mock_delete.mock_calls,
                         [mock.call(mock_create_port())] * ports_per_network)



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import mock
from oslo_config import cfg
import requests

from rally.verification.tempest import config
from tests.unit import fakes
from tests.unit import test

CONF = cfg.CONF


class ConfigTestCase(test.TestCase):

    @mock.patch("rally.objects.deploy.db.deployment_get")
    @mock.patch("rally.osclients.Clients.services",
                return_value={"test_service_type": "test_service"})
    @mock.patch("rally.osclients.Clients.verified_keystone")
    @mock.patch("rally.verification.tempest.config.os.path.isfile",
                return_value=True)
    def setUp(self, mock_isfile, mock_verified_keystone, mock_services,
              mock_get):
        super(ConfigTestCase, self).setUp()
        self.endpoint = {"username": "test",
                         "tenant_name": "test",
                         "password": "test",
                         "auth_url": "http://test/v2.0",
                         "permission": "admin"}
        mock_get.return_value = {"admin": self.endpoint}
        self.deployment = "fake_deployment"
        self.conf_generator = config.TempestConf(self.deployment)
        self.conf_generator.clients.services = mock_services

        keystone_patcher = mock.patch("rally.osclients.create_keystone_client")
        keystone_patcher.start()
        self.addCleanup(keystone_patcher.stop)

    def _remove_default_section(self, items):
        # getting items from configparser by specified section name
        # retruns also values from DEFAULT section
        defaults = (("log_file", "tempest.log"), ("debug", "True"),
                    ("use_stderr", "False"))
        return [item for item in items if item not in defaults]

    @mock.patch("rally.verification.tempest.config.requests")
    @mock.patch("rally.verification.tempest.config.os.rename")
    @mock.patch("six.moves.builtins.open", side_effect=mock.mock_open(),
                create=True)
    def test__load_img_success(self, mock_open, mock_rename, mock_requests):
        mock_result = mock.MagicMock()
        mock_result.status_code = 200
        mock_requests.get.return_value = mock_result
        self.conf_generator._load_img()
        cirros_url = ("http://download.cirros-cloud.net/%s/%s" %
                      (CONF.image.cirros_version,
                       CONF.image.cirros_image))
        mock_requests.get.assert_called_once_with(cirros_url, stream=True)

    @mock.patch("rally.verification.tempest.config.requests")
    def test__load_img_notfound(self, mock_requests):
        mock_result = mock.MagicMock()
        mock_result.status_code = 404
        mock_requests.get.return_value = mock_result
        self.assertRaises(config.TempestConfigCreationFailure,
                          self.conf_generator._load_img)

    def test__get_url(self):
        service = "test_service"
        service_type = "test_service_type"
        url = "test_url"
        # mocked at setUp
        self.conf_generator.keystoneclient.auth_ref = {
            "serviceCatalog": [{
                "name": service,
                "type": service_type,
                "endpoints": [{"publicURL": url}]
            }]}
        self.assertEqual(self.conf_generator._get_url(service), url)

    @mock.patch("rally.verification.tempest.config.TempestConf"
                "._get_url")
    def test__set_boto(self, mock_get_url):
        url = "test_url"
        mock_get_url.return_value = url
        self.conf_generator._set_boto()
        expected = (("ec2_url", url),
                    ("s3_url", url),
                    ("build_interval", "1"),
                    ("build_timeout", "196"),
                    ("http_socket_timeout", "30"),
                    ("instance_type", "m1.nano"),
                    ("ssh_user", "cirros"),
                    ("s3_materials_path",
                     os.path.join(self.conf_generator.data_path,
                                  "s3materials")))
        results = self._remove_default_section(
            self.conf_generator.conf.items("boto"))
        self.assertEqual(sorted(expected), sorted(results))

    def test__set_compute_admin(self):
        self.conf_generator._set_compute_admin()
        expected = [("username", self.endpoint["username"]),
                    ("password", self.endpoint["password"]),
                    ("tenant_name", self.endpoint["tenant_name"])]
        results = self._remove_default_section(
            self.conf_generator.conf.items("compute-admin"))
        self.assertEqual(sorted(expected), sorted(results))

    @mock.patch("rally.osclients.nova")
    def test__set_compute_flavors(self, mock_nova):
        mock_novaclient = mock.MagicMock()
        mock_novaclient.flavors.list.return_value = [
            fakes.FakeFlavor(id="id1"), fakes.FakeFlavor(id="id2")]
        mock_nova.Client.return_value = mock_novaclient
        self.conf_generator._set_compute_flavors()
        expected = ("id1", "id2")
        results = (self.conf_generator.conf.get("compute", "flavor_ref"),
                   self.conf_generator.conf.get("compute", "flavor_ref_alt"))
        self.assertEqual(sorted(expected), sorted(results))

    @mock.patch("rally.osclients.nova")
    def test__set_compute_flavors_create(self, mock_nova):
        mock_novaclient = mock.MagicMock()
        mock_novaclient.flavors.list.return_value = []
        mock_novaclient.flavors.create.side_effect = [
            fakes.FakeFlavor(id="id1"), fakes.FakeFlavor(id="id2")]
        mock_nova.Client.return_value = mock_novaclient
        self.conf_generator._set_compute_flavors()
        self.assertEqual(mock_novaclient.flavors.create.call_count, 2)
        expected = ("id1", "id2")
        results = (self.conf_generator.conf.get("compute", "flavor_ref"),
                   self.conf_generator.conf.get("compute", "flavor_ref_alt"))
        self.assertEqual(sorted(expected), sorted(results))

    @mock.patch("rally.osclients.nova")
    def test__set_compute_flavors_create_fails(self, mock_nova):
        mock_novaclient = mock.MagicMock()
        mock_novaclient.flavors.list.return_value = []
        mock_novaclient.flavors.create.side_effect = Exception()
        mock_nova.Client.return_value = mock_novaclient
        self.assertRaises(config.TempestConfigCreationFailure,
                          self.conf_generator._set_compute_flavors)

    @mock.patch("rally.osclients.glance")
    def test__set_compute_images(self, mock_glance):
        mock_glanceclient = mock.MagicMock()
        mock_glanceclient.images.list.return_value = [
            fakes.FakeImage(id="id1", name="cirros1"),
            fakes.FakeImage(id="id2", name="cirros2")]
        mock_glance.Client.return_value = mock_glanceclient
        self.conf_generator._set_compute_images()
        expected = ("id1", "id2")
        results = (self.conf_generator.conf.get("compute", "image_ref"),
                   self.conf_generator.conf.get("compute", "image_ref_alt"))
        self.assertEqual(sorted(expected), sorted(results))

    @mock.patch("rally.osclients.glance")
    @mock.patch("six.moves.builtins.open")
    def test__set_compute_images_create(self, mock_open, mock_glance):
        mock_glanceclient = mock.MagicMock()
        mock_glanceclient.images.list.return_value = []
        mock_glanceclient.images.create.side_effect = [
            fakes.FakeImage(id="id1"), fakes.FakeImage(id="id2")]
        mock_glance.Client.return_value = mock_glanceclient
        self.conf_generator._set_compute_images()
        self.assertEqual(mock_glanceclient.images.create.call_count, 2)
        expected = ("id1", "id2")
        results = (self.conf_generator.conf.get("compute", "image_ref"),
                   self.conf_generator.conf.get("compute", "image_ref_alt"))
        self.assertEqual(sorted(expected), sorted(results))

    @mock.patch("rally.osclients.glance")
    def test__set_compute_images_create_fails(self, mock_glance):
        mock_glanceclient = mock.MagicMock()
        mock_glanceclient.images.list.return_value = []
        mock_glanceclient.images.create.side_effect = Exception()
        mock_glance.Client.return_value = mock_glanceclient
        self.assertRaises(config.TempestConfigCreationFailure,
                          self.conf_generator._set_compute_images)

    def test__set_compute_ssh_connect_method_if_neutron(self):
        self.conf_generator._set_compute_ssh_connect_method()
        self.assertEqual("fixed",
                         self.conf_generator.conf.get("compute",
                                                      "ssh_connect_method"))
        # if neutron is available
        self.conf_generator.available_services = ["neutron"]
        self.conf_generator._set_compute_ssh_connect_method()
        self.assertEqual("floating",
                         self.conf_generator.conf.get("compute",
                                                      "ssh_connect_method"))

    @mock.patch("rally.verification.tempest.config.os.path.exists",
                return_value=False)
    @mock.patch("rally.verification.tempest.config.os.makedirs")
    def test__set_default(self, mock_makedirs, mock_exists):
        self.conf_generator._set_default()
        lock_path = os.path.join(self.conf_generator.data_path, "lock_files_%s"
                                 % self.deployment)
        mock_makedirs.assert_called_once_with(lock_path)
        expected = (("debug", "True"), ("log_file", "tempest.log"),
                    ("use_stderr", "False"),
                    ("lock_path", lock_path))
        results = self.conf_generator.conf.items("DEFAULT")
        self.assertEqual(sorted(expected), sorted(results))

    def test__set_identity(self):
        self.conf_generator._set_identity()
        expected = (("username", self.endpoint["username"]),
                    ("password", self.endpoint["password"]),
                    ("tenant_name", self.endpoint["tenant_name"]),
                    ("admin_username", self.endpoint["username"]),
                    ("admin_password", self.endpoint["password"]),
                    ("admin_tenant_name", self.endpoint["username"]),
                    ("uri", self.endpoint["auth_url"]),
                    ("uri_v3", self.endpoint["auth_url"].replace("/v2.0",
                                                                 "/v3")))
        results = self._remove_default_section(
            self.conf_generator.conf.items("identity"))
        self.assertEqual(sorted(expected), sorted(results))

    @mock.patch("rally.osclients.neutron")
    def test__set_network_if_neutron(self, mock_neutron):
        fake_neutronclient = mock.MagicMock()
        fake_neutronclient.list_networks.return_value = {"networks": [
                                                         {"status": "ACTIVE",
                                                          "id": "test_id",
                                                          "router:external":
                                                          True}]}
        fake_neutronclient.list_routers.return_value = {"routers": [
                                                        {"id": "test_router"}]}
        fake_neutronclient.list_subnets.return_value = {"subnets": [
                                                        {"cidr":
                                                         "10.0.0.0/24"}]}
        mock_neutron.Client.return_value = fake_neutronclient
        self.conf_generator.available_services = ["neutron"]
        self.conf_generator._set_network()
        expected = (("default_network", "10.0.0.0/24"),
                    ("tenant_networks_reachable", "false"),
                    ("api_version", "2.0"),
                    ("public_network_id", "test_id"),
                    ("public_router_id", "test_router"))
        results = self._remove_default_section(
            self.conf_generator.conf.items("network"))
        self.assertEqual(sorted(expected), sorted(results))

    @mock.patch("rally.osclients.nova")
    def test__set_network_if_nova(self, mock_nova):
        network = "10.0.0.0/24"
        mock_novaclient = mock.MagicMock()
        mock_network = mock.MagicMock()
        mock_network.cidr = network
        mock_novaclient.networks.list.return_value = [mock_network]
        mock_nova.Client.return_value = mock_novaclient
        self.conf_generator._set_network()
        self.assertEqual(network,
                         self.conf_generator.conf.get("network",
                                                      "default_network"))

    @mock.patch("rally.verification.tempest.config.requests")
    def test__set_service_available(self, mock_requests):
        mock_result = mock.MagicMock()
        mock_result.status_code = 404
        mock_requests.get.return_value = mock_result
        available_services = ("nova", "cinder", "glance")
        self.conf_generator.available_services = available_services
        self.conf_generator._set_service_available()
        expected = (("neutron", "False"), ("heat", "False"),
                    ("ceilometer", "False"), ("swift", "False"),
                    ("cinder", "True"), ("nova", "True"),
                    ("glance", "True"), ("horizon", "False"))
        options = self._remove_default_section(
            self.conf_generator.conf.items("service_available"))
        self.assertEqual(sorted(expected), sorted(options))

    @mock.patch("rally.verification.tempest.config.requests")
    def test__set_service_available_horizon(self, mock_requests):
        mock_result = mock.MagicMock()
        mock_result.status_code = 200
        mock_requests.get.return_value = mock_result
        self.conf_generator._set_service_available()
        self.assertEqual(self.conf_generator.conf.get(
            "service_available", "horizon"), "True")

    @mock.patch("rally.verification.tempest.config.requests.get")
    def test__set_service_not_available_horizon(self, mock_requests_get):
        mock_requests_get.side_effect = requests.Timeout()
        self.conf_generator._set_service_available()
        self.assertEqual(self.conf_generator.conf.get(
            "service_available", "horizon"), "False")

    @mock.patch("six.moves.builtins.open", side_effect=mock.mock_open(),
                create=True)
    def test_write_config(self, mock_open):
        self.conf_generator.conf = mock.Mock()
        file_name = "/path/to/fake/conf"

        self.conf_generator.write_config(file_name)

        mock_open.assert_called_once_with(file_name, "w+")
        self.conf_generator.conf.write.assert_called_once_with(
            mock_open.side_effect())



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from rally.benchmark.scenarios.sahara import clusters
from tests.unit import test

SAHARA_CLUSTERS = "rally.benchmark.scenarios.sahara.clusters.SaharaClusters"
SAHARA_UTILS = "rally.benchmark.scenarios.sahara.utils"


class SaharaClustersTestCase(test.TestCase):

    @mock.patch(SAHARA_CLUSTERS + "._delete_cluster")
    @mock.patch(SAHARA_CLUSTERS + "._launch_cluster",
                return_value=mock.MagicMock(id=42))
    @mock.patch(SAHARA_UTILS + ".SaharaScenario.clients")
    def test_create_and_delete_cluster(self, mock_clients, mock_launch_cluster,
                                       mock_delete_cluster):

        clusters_scenario = clusters.SaharaClusters()

        clusters_scenario.context = {
            "tenant": {
                "sahara_image": "test_image"
            }
        }
        clusters_scenario.create_and_delete_cluster(
            flavor="test_flavor",
            workers_count=5,
            plugin_name="test_plugin",
            hadoop_version="test_version")

        mock_launch_cluster.assert_called_once_with(
            flavor_id="test_flavor",
            image_id="test_image",
            workers_count=5,
            plugin_name="test_plugin",
            hadoop_version="test_version",
            floating_ip_pool=None,
            volumes_per_node=None,
            volumes_size=None,
            auto_security_group=None,
            security_groups=None,
            node_configs=None,
            cluster_configs=None)

        mock_delete_cluster.assert_called_once_with(
            mock_launch_cluster.return_value)

    @mock.patch(SAHARA_CLUSTERS + "._delete_cluster")
    @mock.patch(SAHARA_CLUSTERS + "._scale_cluster")
    @mock.patch(SAHARA_CLUSTERS + "._launch_cluster",
                return_value=mock.MagicMock(id=42))
    @mock.patch(SAHARA_UTILS + ".SaharaScenario.clients")
    def test_create_scale_delete_cluster(self, mock_clients,
                                         mock_launch_cluster,
                                         mock_scale_cluster,
                                         mock_delete_cluster):

        mock_sahara = mock_clients("sahara")
        mock_sahara.clusters.get.return_value = mock.MagicMock(
            id=42, status="active"
        )
        clusters_scenario = clusters.SaharaClusters()

        clusters_scenario.context = {
            "tenant": {
                "sahara_image": "test_image"
            }
        }

        clusters_scenario.create_scale_delete_cluster(
            flavor="test_flavor",
            workers_count=5,
            deltas=[1, -1],
            plugin_name="test_plugin",
            hadoop_version="test_version")

        mock_launch_cluster.assert_called_once_with(
            flavor_id="test_flavor",
            image_id="test_image",
            workers_count=5,
            plugin_name="test_plugin",
            hadoop_version="test_version",
            floating_ip_pool=None,
            volumes_per_node=None,
            volumes_size=None,
            auto_security_group=None,
            security_groups=None,
            node_configs=None,
            cluster_configs=None)

        mock_scale_cluster.assert_has_calls([
            mock.call(mock_sahara.clusters.get.return_value, 1),
            mock.call(mock_sahara.clusters.get.return_value, -1),
        ])

        mock_delete_cluster.assert_called_once_with(
            mock_sahara.clusters.get.return_value)



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg

from rally.benchmark.context import base
from rally.benchmark.context.cleanup import manager as resource_manager
from rally.benchmark.scenarios.sahara import utils
from rally.benchmark import utils as bench_utils
from rally.common.i18n import _
from rally.common import log as logging
from rally.common import utils as rutils
from rally import consts
from rally import exceptions
from rally import osclients


CONF = cfg.CONF
LOG = logging.getLogger(__name__)


@base.context(name="sahara_cluster", order=441)
class SaharaCluster(base.Context):
    """Context class for setting up the Cluster an EDP job."""

    CONFIG_SCHEMA = {
        "type": "object",
        "$schema": consts.JSON_SCHEMA,
        "properties": {
            "plugin_name": {
                "type": "string"
            },
            "hadoop_version": {
                "type": "string",
            },
            "workers_count": {
                "type": "integer",
                "minimum": 1
            },
            "flavor_id": {
                "type": "string",
            },
            "floating_ip_pool": {
                "type": "string",
            },
            "volumes_per_node": {
                "type": "integer",
                "minimum": 1
            },
            "volumes_size": {
                "type": "integer",
                "minimum": 1
            },
            "auto_security_group": {
                "type": "boolean",
            },
            "security_groups": {
                "type": "array",
                "items": {
                    "type": "string"
                }
            },
            "node_configs": {
                "type": "object"
            },
            "cluster_configs": {
                "type": "object"
            }
        },
        "additionalProperties": False,
        "required": ["plugin_name", "hadoop_version", "workers_count",
                     "flavor_id"]
    }

    def __init__(self, context):
        super(SaharaCluster, self).__init__(context)
        self.context["sahara_clusters"] = {}

    @rutils.log_task_wrapper(LOG.info, _("Enter context: `Sahara Cluster`"))
    def setup(self):
        wait_dict = dict()

        for user, tenant_id in rutils.iterate_per_tenants(
                self.context["users"]):
            clients = osclients.Clients(user["endpoint"])

            image_id = self.context["tenants"][tenant_id]["sahara_image"]

            floating_ip_pool = self.config.get("floating_ip_pool")

            temporary_context = {"tenant": self.context["tenants"][tenant_id]}
            cluster = utils.SaharaScenario(
                context=temporary_context, clients=clients)._launch_cluster(
                    plugin_name=self.config["plugin_name"],
                    hadoop_version=self.config["hadoop_version"],
                    flavor_id=self.config["flavor_id"],
                    workers_count=self.config["workers_count"],
                    image_id=image_id,
                    floating_ip_pool=floating_ip_pool,
                    volumes_per_node=self.config.get("volumes_per_node"),
                    volumes_size=self.config.get("volumes_size", 1),
                    auto_security_group=self.config.get("auto_security_group",
                                                        True),
                    security_groups=self.config.get("security_groups"),
                    node_configs=self.config.get("node_configs"),
                    cluster_configs=self.config.get("cluster_configs"),
                    wait_active=False)

            self.context["tenants"][tenant_id]["sahara_cluster"] = cluster.id

            # Need to save the client instance to poll for active status
            wait_dict[cluster] = clients.sahara()

        bench_utils.wait_for(
            resource=wait_dict,
            update_resource=self.update_clusters_dict,
            is_ready=self.all_clusters_active,
            timeout=CONF.benchmark.cluster_create_timeout,
            check_interval=CONF.benchmark.cluster_check_interval)

    def update_clusters_dict(self, dct):
        new_dct = dict()
        for cluster, client in dct.items():
            new_cl = client.clusters.get(cluster.id)
            new_dct[new_cl] = client

        return new_dct

    def all_clusters_active(self, dct):
        for cluster, client in dct.items():
            cluster_status = cluster.status.lower()
            if cluster_status == "error":
                raise exceptions.SaharaClusterFailure(
                    name=cluster.name, action="start",
                    reason=cluster.status_description)
            elif cluster_status != "active":
                return False
        return True

    @rutils.log_task_wrapper(LOG.info, _("Exit context: `Sahara Cluster`"))
    def cleanup(self):

        # TODO(boris-42): Delete only resources created by this context
        resource_manager.cleanup(names=["sahara.clusters"],
                                 users=self.context.get("users", []))



# Copyright 2013: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import netaddr
import six

from rally.common.i18n import _
from rally.common import log as logging
from rally.common import utils
from rally.deploy import engine
from rally.deploy.serverprovider import provider
from rally.deploy.serverprovider.providers import lxc
from rally import exceptions
from rally import objects

LOG = logging.getLogger(__name__)
START_SCRIPT = "start.sh"


def get_script_path(name):
    return os.path.join(os.path.abspath(os.path.dirname(__file__)),
                        "lxc", name)


class LxcEngine(engine.EngineFactory):
    """Deploy with other engines in lxc containers.

    Sample configuration:

        {
            "type": "LxcEngine",
            "provider": {
                "type": "DummyProvider",
                "credentials": [{"user": "root", "host": "example.net"}]
            },
            "distribution": "ubuntu",
            "release": "raring",
            "tunnel_to": ["10.10.10.10", "10.10.10.11"],
            "start_lxc_network": "10.1.1.0/24",
            "container_name_prefix": "devstack-node",
            "containers_per_host": 16,
            "start_script": "~/start.sh",
            "engine": { ... }
        }
    """

    CONFIG_SCHEMA = {
        "type": "object",
        "properties": {
            "type": {"type": "string"},
            "distribution": {"type": "string"},
            "release": {"type": "string"},
            "start_lxc_network": {"type": "string",
                                  "pattern": "^(\d+\.){3}\d+\/\d+$"},
            "containers_per_host": {"type": "integer"},
            "tunnel_to": {"type": "array",
                          "elements": {"type": "string",
                                       "pattern": "^(\d+\.){3}\d+$"}},
            "container_name": {"type": "string"},
            "provider": {"type": "object",
                         "properties": {"type": {"type": "string"}}},
        },
        "required": ["type", "containers_per_host", "container_name",
                     "provider"]
    }

    def validate(self):
        super(LxcEngine, self).validate()
        if "start_lxc_network" not in self.config:
            return
        lxc_net = netaddr.IPNetwork(self.config["start_lxc_network"])
        num_containers = self.config["containers_per_host"]
        if lxc_net.size - 3 < num_containers:
            message = _("Network size is not enough for %d hosts.")
            raise exceptions.InvalidConfigException(message % num_containers)

    def _deploy_first(self, lxc_host, name, distribution, release):
        lxc_host.prepare()
        lxc_host.create_container(name, distribution, release)
        lxc_host.start_containers()
        items = six.iteritems(
            lxc_host.get_server_object(name).get_credentials())
        # filter out all keys where value is None
        credentials = dict(filter(lambda x: x[1] is not None, items))
        engine_config = self.config["engine"].copy()
        engine_config["provider"] = {"type": "DummyProvider",
                                     "credentials": [credentials]}
        deployment = objects.Deployment(config=engine_config,
                                        parent_uuid=self.deployment["uuid"])
        deployer = engine.EngineFactory.get_engine(engine_config["name"],
                                                   deployment)
        deployer.deploy()
        lxc_host.stop_containers()

    def _get_provider(self):
        return provider.ProviderFactory.get_provider(self.config["provider"],
                                                     self.deployment)

    @utils.log_deploy_wrapper(LOG.info, _("Create containers on host"))
    def deploy(self):
        name = self.config["container_name"]
        start_script = self.config.get("start_script",
                                       get_script_path(START_SCRIPT))
        distribution = self.config["distribution"]
        release = self.config.get("release", None)
        network = self.config.get("start_lxc_network")
        if network:
            network = netaddr.IPNetwork(network)
        else:
            ip = "0"

        self.provider = self._get_provider()

        for server in self.provider.create_servers():
            config = {"tunnel_to": self.config.get("tunnel_to", [])}
            if network:
                config["network"] = str(network)
                ip = str(network.ip).replace(".", "-")
            else:
                ip = "0"
            name_prefix = "%s-%s" % (name, ip)
            first_name = name_prefix + "-000"
            lxc_host = lxc.LxcHost(server, config)
            self._deploy_first(lxc_host, first_name, distribution, release)
            for i in range(1, self.config["containers_per_host"]):
                clone_name = "%s-%03d" % (name_prefix, i)
                lxc_host.create_clone(clone_name, first_name)
            lxc_host.start_containers()
            info = {"host": server.get_credentials(),
                    "containers": lxc_host.containers,
                    "forwarded_ports": lxc_host._port_cache.items(),
                    "config": config}
            self.deployment.add_resource(provider_name="LxcEngine", info=info)
            for container in lxc_host.get_server_objects():
                container.ssh.run("/bin/sh -e", stdin=open(start_script, "rb"))
            if network:
                network += 1
        return {"admin": objects.Endpoint("", "", "", "")}

    def cleanup(self):
        resources = self.deployment.get_resources()
        for resource in resources:
            server = provider.Server.from_credentials(resource.info["host"])
            lxc_host = lxc.LxcHost(server, resource.info["config"])
            lxc_host.containers = resource.info["containers"]
            lxc_host.destroy_containers()
            lxc_host.destroy_ports(resource.info["forwarded_ports"])
            lxc_host.delete_tunnels()
            self.deployment.delete_resource(resource.id)
        self._get_provider().destroy_servers()



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from rally.benchmark.scenarios import base


class DesignateScenario(base.Scenario):
    """Base class for Designate scenarios with basic atomic actions."""

    RESOURCE_NAME_PREFIX = "rally_"

    @base.atomic_action_timer("designate.create_domain")
    def _create_domain(self, domain=None):
        """Create domain.

        :param domain: dict, POST /v1/domains request options
        :returns: designate domain dict
        """
        domain = domain or {}

        domain.setdefault("email", "root@random.name")
        domain.setdefault("name", "%s.name." % self._generate_random_name())
        return self.clients("designate").domains.create(domain)

    @base.atomic_action_timer("designate.list_domains")
    def _list_domains(self):
        """Return user domain list."""
        return self.clients("designate").domains.list()

    @base.atomic_action_timer("designate.delete_domain")
    def _delete_domain(self, domain_id):
        """Delete designate zone.

        :param domain_id: domain ID
        """
        self.clients("designate").domains.delete(domain_id)

    def _create_record(self, domain, record=None, atomic_action=True):
        """Create a record in a domain.

        :param domain: domain dict
        :param record: record dict
        :param atomic_action: True if the record creation should be tracked
                              as an atomic action
        :returns: Designate record dict
        """
        record = record or {}
        record.setdefault("type", "A")
        record.setdefault("name", "%s.%s" % (self._generate_random_name(),
                                             domain["name"]))
        record.setdefault("data", "10.0.0.1")

        client = self.clients("designate")

        if atomic_action:
            with base.AtomicAction(self, "designate.create_record"):
                return client.records.create(domain["id"], record)

        return client.records.create(domain["id"], record)

    @base.atomic_action_timer("designate.list_records")
    def _list_records(self, domain_id):
        """List domain records.

        :param domain_id: domain ID
        :returns: domain records list
        """
        return self.clients("designate").records.list(domain_id)

    def _delete_record(self, domain_id, record_id, atomic_action=True):
        """Delete a domain record.

        :param domain_id: domain ID
        :param record_id: record ID
        :param atomic_action: True if the record creation should be tracked
                              as an atomic action
        """
        client = self.clients("designate")

        if atomic_action:
            with base.AtomicAction(self, "designate.delete_record"):
                client.records.create(domain_id, record_id)

        client.records.delete(domain_id, record_id)

    @base.atomic_action_timer("designate.create_server")
    def _create_server(self, server=None):
        """Create server.

        :param server: dict, POST /v1/servers request options
        :returns: designate server dict
        """
        server = server or {}

        server.setdefault("name", "name.%s." % self._generate_random_name())
        return self.admin_clients("designate").servers.create(server)

    @base.atomic_action_timer("designate.list_servers")
    def _list_servers(self):
        """Return user server list."""
        return self.admin_clients("designate").servers.list()

    @base.atomic_action_timer("designate.delete_server")
    def _delete_server(self, server_id):
        """Delete Server.

        :param server_id: unicode server ID
        """
        self.admin_clients("designate").servers.delete(server_id)



# Copyright 2015: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from rally.benchmark.scenarios.mistral import utils
from tests.unit import test

MISTRAL_UTILS = "rally.benchmark.scenarios.mistral.utils"


class MistralScenarioTestCase(test.TestCase):

    def _test_atomic_action_timer(self, atomic_actions, name):
        action_duration = atomic_actions.get(name)
        self.assertIsNotNone(action_duration)
        self.assertIsInstance(action_duration, float)

    @mock.patch(MISTRAL_UTILS + ".MistralScenario.clients")
    def test_list_workbooks(self, mock_clients):
        wbs_list = []
        mock_clients("mistral").workbooks.list.return_value = wbs_list
        scenario = utils.MistralScenario()
        return_wbs_list = scenario._list_workbooks()
        self.assertEqual(wbs_list, return_wbs_list)
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "mistral.list_workbooks")

    @mock.patch(MISTRAL_UTILS + ".MistralScenario.clients")
    def test_create_workbook(self, mock_clients):
        definition = "version: \"2.0\"\nname: wb"
        mock_clients("mistral").workbooks.create.return_value = "wb"
        scenario = utils.MistralScenario()
        wb = scenario._create_workbook(definition)
        self.assertEqual("wb", wb)
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "mistral.create_workbook")

    @mock.patch(MISTRAL_UTILS + ".MistralScenario.clients")
    def test_delete_workbook(self, mock_clients):
        wb = mock.Mock()
        wb.name = "wb"
        mock_clients("mistral").workbooks.delete.return_value = "ok"
        scenario = utils.MistralScenario()
        scenario._delete_workbook(wb.name)
        mock_clients("mistral").workbooks.delete.assert_called_once_with(
            wb.name
        )
        self._test_atomic_action_timer(scenario.atomic_actions(),
                                       "mistral.delete_workbook")



# Copyright 2013: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import mock
from oslo_config import fixture
from oslotest import base

from rally import db


class DatabaseFixture(fixture.Config):
    """Create clean DB before starting test."""
    def setUp(self):
        super(DatabaseFixture, self).setUp()
        db_url = os.environ.get("RALLY_UNITTEST_DB_URL", "sqlite://")
        db.db_cleanup()
        self.conf.set_default("connection", db_url, group="database")
        db.db_drop()
        db.db_create()


class TestCase(base.BaseTestCase):
    """Test case base class for all unit tests."""

    def setUp(self):
        super(TestCase, self).setUp()
        self.addCleanup(mock.patch.stopall)

    def _test_atomic_action_timer(self, atomic_actions, name):
        action_duration = atomic_actions.get(name)
        self.assertIsNotNone(action_duration)
        self.assertIsInstance(action_duration, float)

    def assertSequenceEqual(self, iterable_1, iterable_2):
        self.assertEqual(tuple(iterable_1), tuple(iterable_2))


class DBTestCase(TestCase):
    """Base class for tests which use DB."""

    def setUp(self):
        super(TestCase, self).setUp()
        self.useFixture(DatabaseFixture())



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random

from oslo_config import cfg
from oslo_utils import uuidutils
from saharaclient.api import base as sahara_base

from rally.benchmark.scenarios import base
from rally.benchmark import utils as bench_utils
from rally.common import log as logging
from rally import consts
from rally import exceptions

LOG = logging.getLogger(__name__)
CONF = cfg.CONF

SAHARA_TIMEOUT_OPTS = [
    cfg.IntOpt("cluster_create_timeout", default=1800,
               help="A timeout in seconds for a cluster create operation"),
    cfg.IntOpt("cluster_delete_timeout", default=900,
               help="A timeout in seconds for a cluster delete operation"),
    cfg.IntOpt("cluster_check_interval", default=5,
               help="Cluster status polling interval in seconds"),
    cfg.IntOpt("job_execution_timeout", default=600,
               help="A timeout in seconds for a cluster create operation"),
    cfg.IntOpt("job_check_interval", default=5,
               help="Cluster status polling interval in seconds")
]

benchmark_group = cfg.OptGroup(name="benchmark", title="benchmark options")
CONF.register_opts(SAHARA_TIMEOUT_OPTS, group=benchmark_group)


class SaharaScenario(base.Scenario):
    """Base class for Sahara scenarios with basic atomic actions."""

    RESOURCE_NAME_LENGTH = 20

    # TODO(nkonovalov): Add other provisioning plugins
    NODE_PROCESSES = {
        "vanilla": {
            "1.2.1": {
                "master": ["namenode", "jobtracker", "oozie"],
                "worker": ["datanode", "tasktracker"]
            },
            "2.3.0": {
                "master": ["namenode", "resourcemanager", "historyserver",
                           "oozie"],
                "worker": ["datanode", "nodemanager"]
            },
            "2.4.1": {
                "master": ["namenode", "resourcemanager", "historyserver",
                           "oozie"],
                "worker": ["datanode", "nodemanager"]
            }
        },
        "hdp": {
            "1.3.2": {
                "master": ["JOBTRACKER", "NAMENODE", "SECONDARY_NAMENODE",
                           "GANGLIA_SERVER", "NAGIOS_SERVER",
                           "AMBARI_SERVER", "OOZIE_SERVER"],
                "worker": ["TASKTRACKER", "DATANODE", "HDFS_CLIENT",
                           "MAPREDUCE_CLIENT", "OOZIE_CLIENT", "PIG"]
            },
            "2.0.6": {
                "manager": ["AMBARI_SERVER", "GANGLIA_SERVER",
                            "NAGIOS_SERVER"],
                "master": ["NAMENODE", "SECONDARY_NAMENODE",
                           "ZOOKEEPER_SERVER", "ZOOKEEPER_CLIENT",
                           "HISTORYSERVER", "RESOURCEMANAGER",
                           "OOZIE_SERVER"],
                "worker": ["DATANODE", "HDFS_CLIENT", "ZOOKEEPER_CLIENT",
                           "PIG", "MAPREDUCE2_CLIENT", "YARN_CLIENT",
                           "NODEMANAGER", "OOZIE_CLIENT"]
            }
        },
        "cdh": {
            "5": {
                "manager": ["MANAGER"],
                "master": ["NAMENODE", "SECONDARYNAMENODE", "RESOURCEMANAGER",
                           "JOBHISTORY", "OOZIE_SERVER"],
                "worker": ["DATANODE", "NODEMANAGER"]
            }
        }
    }

    REPLICATION_CONFIGS = {
        "vanilla": {
            "1.2.1": {
                "target": "HDFS",
                "config_name": "dfs.replication"
            },
            "2.3.0": {
                "target": "HDFS",
                "config_name": "dfs.replication"
            },
            "2.4.1": {
                "target": "HDFS",
                "config_name": "dfs.replication"
            }
        },
        "hdp": {
            "1.3.2": {
                "target": "HDFS",
                "config_name": "dfs.replication"
            },
            "2.0.6": {
                "target": "HDFS",
                "config_name": "dfs.replication"
            }
        },
        "cdh": {
            "5": {
                "target": "HDFS",
                "config_name": "dfs_replication"
            }
        }
    }

    @base.atomic_action_timer("sahara.list_node_group_templates")
    def _list_node_group_templates(self):
        """Return user Node Group Templates list."""
        return self.clients("sahara").node_group_templates.list()

    @base.atomic_action_timer("sahara.create_master_node_group_template")
    def _create_master_node_group_template(self, flavor_id, plugin_name,
                                           hadoop_version):
        """Create a master Node Group Template with a random name.

        :param flavor_id: The required argument for the Template
        :param plugin_name: Sahara provisioning plugin name
        :param hadoop_version: The version of Hadoop distribution supported by
                               the plugin
        :returns: The created Template
        """
        name = self._generate_random_name(prefix="master-ngt-")

        return self.clients("sahara").node_group_templates.create(
            name=name,
            plugin_name=plugin_name,
            hadoop_version=hadoop_version,
            flavor_id=flavor_id,
            node_processes=self.NODE_PROCESSES[plugin_name][hadoop_version]
            ["master"])

    @base.atomic_action_timer("sahara.create_worker_node_group_template")
    def _create_worker_node_group_template(self, flavor_id, plugin_name,
                                           hadoop_version):
        """Create a worker Node Group Template with a random name.

        :param flavor_id: The required argument for the Template
        :param plugin_name: Sahara provisioning plugin name
        :param hadoop_version: The version of Hadoop distribution supported by
                               the plugin
        :returns: The created Template
        """
        name = self._generate_random_name(prefix="worker-ngt-")

        return self.clients("sahara").node_group_templates.create(
            name=name,
            plugin_name=plugin_name,
            hadoop_version=hadoop_version,
            flavor_id=flavor_id,
            node_processes=self.NODE_PROCESSES[plugin_name][hadoop_version]
            ["worker"])

    @base.atomic_action_timer("sahara.delete_node_group_template")
    def _delete_node_group_template(self, node_group):
        """Delete a Node Group Template by id.

        :param node_group: The Node Group Template to be deleted
        """
        self.clients("sahara").node_group_templates.delete(node_group.id)

    def _wait_active(self, cluster_object):
        bench_utils.wait_for(
            resource=cluster_object, is_ready=self._is_cluster_active,
            update_resource=self._update_cluster,
            timeout=CONF.benchmark.cluster_create_timeout,
            check_interval=CONF.benchmark.cluster_check_interval)

    def _setup_neutron_floating_ip_pool(self, name_or_id):
        if name_or_id:
            if uuidutils.is_uuid_like(name_or_id):
                # Looks like an id is provided Return as is.
                return name_or_id
            else:
                # It's a name. Changing to id.
                for net in self.clients("neutron").list_networks()["networks"]:
                    if net["name"] == name_or_id:
                        return net["id"]
                # If the name is not found in the list. Exit with error.
                raise exceptions.BenchmarkSetupFailure(
                    "Could not resolve Floating IP Pool name %(name)s to id" %
                    name_or_id)
        else:
            # Pool is not provided. Using the one set as GW for current router.

            net = self.context["tenant"]["networks"][0]
            router_id = net["router_id"]
            router = self.clients("neutron").show_router(router_id)["router"]
            net_id = router["external_gateway_info"]["network_id"]

            return net_id

    def _setup_nova_floating_ip_pool(self, name):
        if name:
            # The name is provided returning it as is.
            return name
        else:
            # The name is not provided. Discovering
            LOG.debug("No Floating Ip Pool provided. Taking random.")
            pools = self.clients("nova").floating_ip_pools.list()

            if pools:
                return random.choice(pools).name
            else:
                LOG.warn("No Floating Ip Pools found. This may cause "
                         "instances to be unreachable.")
                return None

    def _setup_floating_ip_pool(self, node_groups, floating_ip_pool):
        if consts.Service.NEUTRON in self._clients.services().values():
            LOG.debug("Neutron detected as networking backend.")
            floating_ip_pool_value = self._setup_neutron_floating_ip_pool(
                floating_ip_pool)
        else:
            LOG.debug("Nova Network detected as networking backend.")
            floating_ip_pool_value = self._setup_nova_floating_ip_pool(
                floating_ip_pool)

        if floating_ip_pool_value:
            LOG.debug("Using floating ip pool %s." % floating_ip_pool_value)
            # If the pool is set by any means assign it to all node groups.
            for ng in node_groups:
                ng["floating_ip_pool"] = floating_ip_pool_value

        return node_groups

    def _setup_volumes(self, node_groups, volumes_per_node, volumes_size):
        if volumes_per_node:
            LOG.debug("Adding volumes config to Node Groups")
            for ng in node_groups:
                ng["volumes_per_node"] = volumes_per_node
                ng["volumes_size"] = volumes_size

        return node_groups

    def _setup_security_groups(self, node_groups, auto_security_group,
                               security_groups):
        if auto_security_group:
            LOG.debug("Auto security group enabled. Adding to Node Groups.")
        if security_groups:
            LOG.debug("Adding provided Security Groups to Node Groups.")

        for ng in node_groups:
            if auto_security_group:
                ng["auto_security_group"] = auto_security_group
            if security_groups:
                ng["security_groups"] = security_groups

        return node_groups

    def _setup_node_configs(self, node_groups, node_configs):
        if node_configs:
            LOG.debug("Adding Hadoop configs to Node Groups")
            for ng in node_groups:
                ng["node_configs"] = node_configs

        return node_groups

    def _setup_replication_config(self, hadoop_version, workers_count,
                                  plugin_name):
        replication_value = min(workers_count, 3)
        # 3 is a default Hadoop replication
        conf = self.REPLICATION_CONFIGS[plugin_name][hadoop_version]
        LOG.debug("Using replication factor: %s" % replication_value)
        replication_config = {
            conf["target"]: {
                conf["config_name"]: replication_value
            }
        }
        return replication_config

    @base.atomic_action_timer("sahara.launch_cluster")
    def _launch_cluster(self, plugin_name, hadoop_version, flavor_id,
                        image_id, workers_count, floating_ip_pool=None,
                        volumes_per_node=None,
                        volumes_size=None, auto_security_group=None,
                        security_groups=None, node_configs=None,
                        cluster_configs=None, wait_active=True):
        """Create a cluster and wait until it becomes Active.

        The cluster is created with two node groups. The master Node Group is
        created with one instance. The worker node group contains
        node_count - 1 instances.

        :param plugin_name: provisioning plugin name
        :param hadoop_version: Hadoop version supported by the plugin
        :param flavor_id: flavor which will be used to create instances
        :param image_id: image id that will be used to boot instances
        :param workers_count: number of worker instances. All plugins will
                              also add one Master instance and some plugins
                              add a Manager instance.
        :param floating_ip_pool: floating ip pool name from which Floating
                                 IPs will be allocated
        :param volumes_per_node: number of Cinder volumes that will be
                                 attached to every cluster node
        :param volumes_size: size of each Cinder volume in GB
        :param auto_security_group: boolean value. If set to True Sahara will
                                    create a Security Group for each Node Group
                                    in the Cluster automatically.
        :param security_groups: list of security groups that will be used
                                while creating VMs. If auto_security_group is
                                set to True, this list can be left empty.
        :param node_configs: configs dict that will be passed to each Node
                             Group
        :param cluster_configs: configs dict that will be passed to the
                                Cluster
        :param wait_active: Wait until a Cluster gets int "Active" state
        :returns: created cluster
        """
        node_groups = [
            {
                "name": "master-ng",
                "flavor_id": flavor_id,
                "node_processes": self.NODE_PROCESSES[plugin_name]
                [hadoop_version]["master"],
                "count": 1
            }, {
                "name": "worker-ng",
                "flavor_id": flavor_id,
                "node_processes": self.NODE_PROCESSES[plugin_name]
                [hadoop_version]["worker"],
                "count": workers_count
            }
        ]

        if "manager" in self.NODE_PROCESSES[plugin_name][hadoop_version]:
            # Adding manager group separately as it is supported only in
            # specific configurations.

            node_groups.append({
                "name": "manager-ng",
                "flavor_id": flavor_id,
                "node_processes": self.NODE_PROCESSES[plugin_name]
                [hadoop_version]["manager"],
                "count": 1
            })

        node_groups = self._setup_floating_ip_pool(node_groups,
                                                   floating_ip_pool)

        neutron_net_id = self._get_neutron_net_id()

        node_groups = self._setup_volumes(node_groups, volumes_per_node,
                                          volumes_size)

        node_groups = self._setup_security_groups(node_groups,
                                                  auto_security_group,
                                                  security_groups)

        node_groups = self._setup_node_configs(node_groups, node_configs)

        replication_config = self._setup_replication_config(hadoop_version,
                                                            workers_count,
                                                            plugin_name)

        # The replication factor should be set for small clusters. However the
        # cluster_configs parameter can override it
        merged_cluster_configs = self._merge_configs(replication_config,
                                                     cluster_configs)

        name = self._generate_random_name(prefix="sahara-cluster-")

        cluster_object = self.clients("sahara").clusters.create(
            name=name,
            plugin_name=plugin_name,
            hadoop_version=hadoop_version,
            node_groups=node_groups,
            default_image_id=image_id,
            net_id=neutron_net_id,
            cluster_configs=merged_cluster_configs
        )

        if wait_active:
            self._wait_active(cluster_object)

        return self.clients("sahara").clusters.get(cluster_object.id)

    def _update_cluster(self, cluster):
        return self.clients("sahara").clusters.get(cluster.id)

    def _is_cluster_active(self, cluster):
        cluster_status = cluster.status.lower()

        if cluster_status == "error":
            raise exceptions.SaharaClusterFailure(
                name=cluster.name,
                action="start",
                reason=cluster.status_description)

        return cluster_status == "active"

    def _scale_cluster(self, cluster, delta):
        """The scaling helper.

        This method finds the worker node group in a cluster, builds a
        scale_object required by Sahara API and waits for the scaling to
        complete.

        NOTE: This method is not meant to be called directly in benchmarks.
        There two specific scaling methods of up and down scaling which have
        different atomic timers.
        """
        worker_node_group = [g for g in cluster.node_groups
                             if "worker" in g["name"]][0]
        scale_object = {
            "resize_node_groups": [
                {
                    "name": worker_node_group["name"],
                    "count": worker_node_group["count"] + delta
                }
            ]
        }
        self.clients("sahara").clusters.scale(cluster.id, scale_object)

        self._wait_active(cluster)

    @base.atomic_action_timer("sahara.scale_up")
    def _scale_cluster_up(self, cluster, delta):
        """Add a given number of worker nodes to the cluster.

        :param cluster: The cluster to be scaled
        :param delta: The number of workers to be added. (A positive number is
                      expected here)
        """
        self._scale_cluster(cluster, delta)

    @base.atomic_action_timer("sahara.scale_down")
    def _scale_cluster_down(self, cluster, delta):
        """Remove a given number of worker nodes from the cluster.

        :param cluster: The cluster to be scaled
        :param delta: The number of workers to be removed. (A negative number
                      is expected here)
        """
        self._scale_cluster(cluster, delta)

    @base.atomic_action_timer("sahara.delete_cluster")
    def _delete_cluster(self, cluster):
        """Delete cluster.

        :param cluster: cluster to delete
        """
        self.clients("sahara").clusters.delete(cluster.id)

        bench_utils.wait_for(
            resource=cluster.id,
            timeout=CONF.benchmark.cluster_delete_timeout,
            check_interval=CONF.benchmark.cluster_check_interval,
            is_ready=self._is_cluster_deleted)

    def _is_cluster_deleted(self, cl_id):
        try:
            self.clients("sahara").clusters.get(cl_id)
            return False
        except sahara_base.APIException:
            return True

    def _create_output_ds(self):
        """Create an output Data Source based on EDP context

        :return: The created Data Source
        """
        ds_type = self.context["sahara_output_conf"]["output_type"]
        url_prefix = self.context["sahara_output_conf"]["output_url_prefix"]

        if ds_type == "swift":
            raise exceptions.RallyException(
                _("Swift Data Sources are not implemented yet"))

        url = (url_prefix.rstrip("/") + "/%s" %
               self._generate_random_name(length=10))

        return self.clients("sahara").data_sources.create(
            name=self._generate_random_name(prefix="out_"),
            description="",
            data_source_type=ds_type,
            url=url)

    def _run_job_execution(self, job_id, cluster_id, input_id, output_id,
                           configs, job_idx):
        """Run a Job Execution and wait until it completes or fails.

        The Job Execution is accepted as successful when Oozie reports
        "success" or "succeeded" status. The failure statuses are "failed" and
        "killed".

        The timeout and the polling interval may be configured through
        "job_execution_timeout" and "job_check_interval" parameters under the
        "benchmark" section.

        :param job_id: The Job id that will be executed
        :param cluster_id: The Cluster id which will execute the Job
        :param input_id: The input Data Source id
        :param output_id: The output Data Source id
        :param configs: The config dict that will be passed as Job Execution's
                        parameters.
        :param job_idx: The index of a job in a sequence

        """
        @base.atomic_action_timer("sahara.job_execution_%s" % job_idx)
        def run(self):
            job_execution = self.clients("sahara").job_executions.create(
                job_id=job_id,
                cluster_id=cluster_id,
                input_id=input_id,
                output_id=output_id,
                configs=configs)

            bench_utils.wait_for(
                resource=job_execution.id,
                is_ready=self._job_execution_is_finished,
                timeout=CONF.benchmark.job_execution_timeout,
                check_interval=CONF.benchmark.job_check_interval)

        run(self)

    def _job_execution_is_finished(self, je_id):
        status = self.clients("sahara").job_executions.get(je_id).info[
            "status"]
        if status.lower() in ("success", "succeeded"):
            return True
        elif status.lower() in ("failed", "killed"):
            raise exceptions.RallyException(
                "Job execution %s has failed" % je_id)
        return False

    def _merge_configs(self, *configs):
        """Merge configs in special format.

        It supports merging of configs in the following format:
        applicable_target -> config_name -> config_value

        """
        result = {}
        for config_dict in configs:
            if config_dict:
                for a_target in config_dict:
                    if a_target not in result or not result[a_target]:
                        result[a_target] = {}
                    result[a_target].update(config_dict[a_target])

        return result

    def _get_neutron_net_id(self):
        """Get the Neutron Network id from context.

        If Nova Network is used as networking backend, None is returned.

        :return: Network id for Neutron or None for Nova Networking.
        """

        if consts.Service.NEUTRON not in self._clients.services().values():
            return None

        # Taking net id from context.
        net = self.context["tenant"]["networks"][0]
        neutron_net_id = net["id"]
        LOG.debug("Using neutron network %s." % neutron_net_id)
        LOG.debug("Using neutron router %s." % net["router_id"])

        return neutron_net_id



# Copyright 2013: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import jsonschema
import mock
import netaddr
from oslotest import mockpatch

from rally.deploy.serverprovider.providers import virsh
from tests.unit import test


class VirshProviderTestCase(test.TestCase):
    def setUp(self):
        super(VirshProviderTestCase, self).setUp()
        self.deployment = mock.Mock()
        self.config = {
            "type": "VirshProvider",
            "connection": "user@host",
            "template_name": "prefix",
            "template_user": "user",
            "template_password": "password",
        }
        self.provider = virsh.VirshProvider(self.deployment, self.config)
        self.useFixture(mockpatch.PatchObject(self.provider, "resources"))

    @mock.patch(
        "rally.deploy.serverprovider.providers.virsh.netaddr.IPAddress")
    @mock.patch("rally.deploy.serverprovider.providers.virsh.subprocess")
    @mock.patch("time.sleep")
    def test_create_vm(self, mock_sleep, mock_subp, mock_ipaddress):
        mock_subp.check_output.return_value = "10.0.0.1"
        mock_ipaddress.return_value = "10.0.0.2"
        server = self.provider.create_vm("name")
        script_path = "%(virsh_path)s/virsh/get_domain_ip.sh" % dict(
                virsh_path=os.path.split(virsh.__file__)[0])
        mock_subp.assert_has_calls([
            mock.call.check_call("virt-clone --connect=qemu+ssh://user@host/"
                                 "system -o prefix -n name --auto-clone",
                                 shell=True),
            mock.call.check_call("virsh --connect=qemu+ssh://user@host/system "
                                 "start name", shell=True),
            mock.call.check_call("scp -o StrictHostKeyChecking=no  %s u"
                                 "ser@host:~/get_domain_ip.sh" % script_path,
                                 shell=True),
            mock.call.check_output("ssh -o StrictHostKeyChecking=no user@host "
                                   "./get_domain_ip.sh name", shell=True),
        ])
        mock_ipaddress.assert_called_once_with("10.0.0.1")
        self.assertEqual(server.host, "10.0.0.2")
        self.assertEqual(server.user, "user")
        self.assertIsNone(server.key)
        self.assertEqual(server.password, "password")
        self.provider.resources.create.assert_called_once_with({
            "name": "name",
        })

    @mock.patch(
        "rally.deploy.serverprovider.providers.virsh.netaddr.IPAddress")
    @mock.patch("rally.deploy.serverprovider.providers.virsh.subprocess")
    @mock.patch("time.sleep")
    def test_create_vm_ip_failed(self, mock_sleep, mock_subp, mock_ipaddress):
        mock_ipaddress.side_effect = netaddr.core.AddrFormatError
        server = self.provider.create_vm("name")
        mock_subp.assert_has_calls(3 * [
            mock.call.check_output("ssh -o StrictHostKeyChecking=no user@host "
                                   "./get_domain_ip.sh name", shell=True),
        ])
        self.assertEqual(server.host, "None")

    @mock.patch("rally.deploy.serverprovider.providers.virsh.subprocess")
    def test_destroy_vm(self, mock_subp):
        self.provider.destroy_vm("uuid")
        mock_subp.assert_has_calls([
            mock.call.check_call("virsh --connect=qemu+ssh://user@host/system "
                                 "destroy uuid", shell=True),
            mock.call.check_call("virsh --connect=qemu+ssh://user@host/system "
                                 "undefine uuid --remove-all-storage",
                                 shell=True),
        ])

    @mock.patch("rally.deploy.serverprovider.providers.virsh.uuid")
    @mock.patch.object(virsh.VirshProvider, "create_vm")
    def test_create_servers(self, mock_create, mock_uuid):
        mock_uuid.uuid4.side_effect = ["1", "2", "3"]
        mock_create.side_effect = ["s1", "s2", "s3"]
        servers = self.provider.create_servers(amount=3)
        self.assertEqual(servers, ["s1", "s2", "s3"])
        mock_create.assert_has_calls([
            mock.call("1"),
            mock.call("2"),
            mock.call("3"),
        ])

    @mock.patch.object(virsh.VirshProvider, "destroy_vm")
    def test_destroy_servers(self, mock_destroy):
        self.provider.resources.get_all.return_value = [
            {"info": {"name": "1"}},
            {"info": {"name": "2"}},
            {"info": {"name": "3"}},
        ]
        self.provider.destroy_servers()
        mock_destroy.assert_has_calls([
            mock.call("1"),
            mock.call("2"),
            mock.call("3"),
        ])
        self.provider.resources.get_all.assert_called_once_with()

    def test_invalid_config(self):
        self.config["type"] = 42
        self.assertRaises(jsonschema.ValidationError, virsh.VirshProvider,
                          self.deployment, self.config)

    def test_invalid_connection(self):
        self.config["connection"] = "user host"
        self.assertRaises(jsonschema.ValidationError, virsh.VirshProvider,
                          self.deployment, self.config)



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import jsonschema
import mock

from rally.benchmark.context.cleanup import context
from tests.unit import test


BASE = "rally.benchmark.context.cleanup.context"


class AdminCleanupTestCase(test.TestCase):

    @mock.patch("%s.manager" % BASE)
    def test_validate(self, mock_manager):
        mock_manager.list_resource_names.return_value = set(["a", "b", "c"])
        context.AdminCleanup.validate(["a"])
        mock_manager.list_resource_names.assert_called_once_with(
            admin_required=True)

    @mock.patch("%s.manager" % BASE)
    def test_validate_no_such_cleanup(self, mock_manager):
        mock_manager.list_resource_names.return_value = set(["a", "b", "c"])
        self.assertRaises(context.NoSuchCleanupResources,
                          context.AdminCleanup.validate, ["a", "d"])
        mock_manager.list_resource_names.assert_called_once_with(
            admin_required=True)

    def test_validate_invalid_config(self):
        self.assertRaises(jsonschema.ValidationError,
                          context.AdminCleanup.validate, {})

    @mock.patch("%s.manager.find_resource_managers" % BASE,
                return_value=[mock.MagicMock(), mock.MagicMock()])
    @mock.patch("%s.manager.SeekAndDestroy" % BASE)
    def test_cleanup(self, mock_seek_and_destroy, mock_find_res_mgr):

        ctx = {
            "config": {"admin_cleanup": ["a", "b"]},
            "admin": mock.MagicMock(),
            "users": mock.MagicMock(),
            "task": mock.MagicMock()
        }

        admin_cleanup = context.AdminCleanup(ctx)
        admin_cleanup.setup()
        admin_cleanup.cleanup()

        mock_find_res_mgr.assert_called_once_with(["a", "b"], True)
        mock_seek_and_destroy.assert_has_calls([
            mock.call(mock_find_res_mgr.return_value[0], ctx["admin"],
                      ctx["users"]),
            mock.call().exterminate(),
            mock.call(mock_find_res_mgr.return_value[1], ctx["admin"],
                      ctx["users"]),
            mock.call().exterminate()
        ])


class UserCleanupTestCase(test.TestCase):

    @mock.patch("%s.manager" % BASE)
    def test_validate(self, mock_manager):
        mock_manager.list_resource_names.return_value = set(["a", "b", "c"])
        context.UserCleanup.validate(["a"])
        mock_manager.list_resource_names.assert_called_once_with(
                admin_required=False)

    @mock.patch("%s.manager" % BASE)
    def test_validate_no_such_cleanup(self, mock_manager):
        mock_manager.list_resource_names.return_value = set(["a", "b", "c"])
        self.assertRaises(context.NoSuchCleanupResources,
                          context.UserCleanup.validate, ["a", "b", "d"])
        mock_manager.list_resource_names.assert_called_once_with(
                admin_required=False)

    def test_validate_invalid_config(self):
        self.assertRaises(jsonschema.ValidationError,
                          context.UserCleanup.validate, {})

    @mock.patch("%s.manager.find_resource_managers" % BASE,
                return_value=[mock.MagicMock(), mock.MagicMock()])
    @mock.patch("%s.manager.SeekAndDestroy" % BASE)
    def test_cleanup(self, mock_seek_and_destroy, mock_find_res_mgr):

        ctx = {
            "config": {"cleanup": ["a", "b"]},
            "users": mock.MagicMock(),
            "task": mock.MagicMock()
        }

        admin_cleanup = context.UserCleanup(ctx)
        admin_cleanup.setup()
        admin_cleanup.cleanup()

        mock_find_res_mgr.assert_called_once_with(["a", "b"], False)

        mock_seek_and_destroy.assert_has_calls([
            mock.call(mock_find_res_mgr.return_value[0], None, ctx["users"]),
            mock.call().exterminate(),
            mock.call(mock_find_res_mgr.return_value[1], None, ctx["users"]),
            mock.call().exterminate()
        ])



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from rally.benchmark.processing import utils
from rally import exceptions
from tests.unit import test


class MathTestCase(test.TestCase):

    def test_percentile(self):
        lst = list(range(1, 101))
        result = utils.percentile(lst, 0.1)
        self.assertEqual(result, 10.9)

    def test_percentile_value_none(self):
        result = utils.percentile(None, 0.1)
        self.assertIsNone(result)

    def test_percentile_equal(self):
        lst = list(range(1, 101))
        result = utils.percentile(lst, 1)
        self.assertEqual(result, 100)

    def test_mean(self):
        lst = list(range(1, 100))
        result = utils.mean(lst)
        self.assertEqual(result, 50.0)

    def test_mean_empty_list(self):
        lst = []
        self.assertRaises(exceptions.InvalidArgumentsException,
                          utils.mean, lst)

    def _compare_items_lists(self, list1, list2):
        """Items lists comparison, compatible with Python 2.6/2.7.

        :param list1: items list [(a1, b1), (a2, b2) ...]
        :param list2: items list [(a1, b1), (a2, b2) ...]
        """
        compare_float = lambda f1, f2: abs(f2 - f2) < 0.1
        for a, b in zip(list1, list2):
            a1, a2, b1, b2 = (a + b)
            self.assertEqual(a1, b1)
            if type(a2) is float:
                # Float representation is defferent in Python 2.6/2.7,
                # so we need to be sure that values are close to each other
                self.assertTrue(compare_float(a2, b2))
            else:
                self.assertEqual(a2, b2)

    def test_compress(self):
        data64 = range(64)
        data4 = [4, 2, 1, 3]
        mixed = [2, "5", None, 0.5]
        alt_normalize = str
        alt_merge = lambda a, b: str(a) + str(b)
        compress = lambda lst: [(k + 1, float(v)) for k, v in enumerate(lst)]

        # Long list
        self.assertEqual(utils.compress(data64), compress(data64))
        self._compare_items_lists(
            utils.compress(data64, limit=4),
            [(17, 15.0), (33, 31.01), (49, 47.0), (64, 62.0)])
        self.assertEqual(
            utils.compress(data64, limit=4,
                           normalize=alt_normalize, merge=alt_merge),
            [(17, "012345678910111213141516"),
             (33, "17181920212223242526272829303132"),
             (49, "33343536373839404142434445464748"),
             (64, "495051525354555657585960616263")])

        # Short list
        self.assertEqual(utils.compress(data4, limit=2),
                         [(3, 2.0), (4, 3.0)])
        self.assertEqual(utils.compress(data4, normalize=alt_normalize),
                         [(1, "4"), (2, "2"), (3, "1"), (4, "3")])

        # List with mixed data types
        self.assertEqual(utils.compress(mixed),
                         [(1, 2.0), (2, 5.0), (3, 0.0), (4, 0.5)])
        self.assertEqual(utils.compress(mixed, normalize=str),
                         [(1, "2"), (2, "5"), (3, "None"), (4, "0.5")])
        self.assertRaises(TypeError, utils.compress, mixed, normalize=int)
        self.assertEqual(
            utils.compress(mixed, normalize=alt_normalize, merge=alt_merge),
            [(1, "2"), (2, "5"), (3, "None"), (4, "0.5")])


class AtomicActionsDataTestCase(test.TestCase):

    def test_get_atomic_actions_data(self):
        raw_data = [
            {
                "error": [],
                "duration": 3,
                "atomic_actions": {
                    "action1": 1,
                    "action2": 2
                }
            },
            {
                "error": ["some", "error", "occurred"],
                "duration": 1.9,
                "atomic_actions": {
                    "action1": 0.5,
                    "action2": 1.4
                }
            },
            {
                "error": [],
                "duration": 8,
                "atomic_actions": {
                    "action1": 4,
                    "action2": 4
                }
            }
        ]

        atomic_actions_data = {
            "action1": [1, 0.5, 4],
            "action2": [2, 1.4, 4],
            "total": [3, 8]
        }

        output = utils.get_atomic_actions_data(raw_data)
        self.assertEqual(output, atomic_actions_data)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from rally.verification.tempest import compare2html
from tests.unit import test


class Compare2HtmlTestCase(test.TestCase):

    def test_main(self):
        results = [{"val2": 0.0111, "field": u"time", "val1": 0.0222,
                    "type": "CHANGED", "test_name": u"test.one"},
                   {"val2": 0.111, "field": u"time", "val1": 0.222,
                    "type": "CHANGED", "test_name": u"test.two"},
                   {"val2": 1.11, "field": u"time", "val1": 2.22,
                    "type": "CHANGED", "test_name": u"test.three"}]

        fake_kw = {"heading":
                   {"title": compare2html.__title__,
                    "description": compare2html.__description__,
                    "parameters": [("Difference Count", len(results))]
                    },
                   "generator": "compare2html %s" % compare2html.__version__,
                   "results": results}

        with mock.patch("mako.template.Template") as mock_mako:
            compare2html.create_report(results)
            mock_mako().render.assert_called_once_with(**fake_kw)



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from neutronclient.common import exceptions as neutron_exceptions

from rally.benchmark.context.cleanup import base
from rally.benchmark.context.cleanup import resources
from rally.benchmark.scenarios.keystone import utils as keystone_utils
from rally.common import utils
from tests.unit import test

BASE = "rally.benchmark.context.cleanup.resources"


class AllResourceManagerTestCase(test.TestCase):

    def test_res_manager_special_field(self):

        for res_mgr in utils.itersubclasses(base.ResourceManager):
            manager_name = "%s.%s" % (res_mgr.__module__, res_mgr.__name__)

            fields = filter(lambda x: not x.startswith("__"), dir(res_mgr))

            available_opts = set([
                "_admin_required", "_perform_for_admin_only",
                "_tenant_resource", "_service", "_resource", "_order",
                "_max_attempts", "_timeout", "_interval", "_threads",
                "_manager", "id", "is_deleted", "delete", "list"
            ])

            extra_opts = set(fields) - available_opts

            self.assertFalse(
                extra_opts,
                ("ResourceManager %(name)s contains extra fields: %(opts)s."
                 " Remove them to pass this test")
                % {"name": manager_name, "opts": ", ".join(extra_opts)})


class SynchronizedDeletionTestCase(test.TestCase):

    def test_is_deleted(self):
        self.assertTrue(resources.SynchronizedDeletion().is_deleted())


class QuotaMixinTestCase(test.TestCase):

    def test_id(self):
        quota = resources.QuotaMixin()
        quota.raw_resource = mock.MagicMock()
        self.assertEqual(quota.raw_resource, quota.id())

    def test_delete(self):
        quota = resources.QuotaMixin()
        mock_manager = mock.MagicMock()
        quota._manager = lambda: mock_manager
        quota.raw_resource = mock.MagicMock()

        quota.delete()
        mock_manager.delete.assert_called_once_with(quota.raw_resource)

    def test_list(self):
        quota = resources.QuotaMixin()
        quota.tenant_uuid = None
        self.assertEqual([], quota.list())

        quota.tenant_uuid = mock.MagicMock()
        self.assertEqual([quota.tenant_uuid], quota.list())


class NovaSecurityGroupTestCase(test.TestCase):

    @mock.patch("%s.base.ResourceManager._manager" % BASE)
    def test_list(self, mock_manager):
        secgroups = [mock.MagicMock(), mock.MagicMock(), mock.MagicMock()]
        secgroups[0].name = "a"
        secgroups[1].name = "b"
        secgroups[2].name = "default"

        mock_manager().list.return_value = secgroups
        self.assertSequenceEqual(secgroups[:2],
                                 resources.NovaSecurityGroup().list())


class NeutronMixinTestCase(test.TestCase):

    def get_neutron_mixin(self):
        neut = resources.NeutronMixin()
        neut._service = "neutron"
        return neut

    def test_manager(self):
        neut = self.get_neutron_mixin()
        neut.user = mock.MagicMock()
        self.assertEqual(neut.user.neutron.return_value, neut._manager())

    def test_id(self):
        neut = self.get_neutron_mixin()
        neut.raw_resource = {"id": "test"}
        self.assertEqual("test", neut.id())

    def test_delete(self):
        neut = self.get_neutron_mixin()
        neut.user = mock.MagicMock()
        neut._resource = "some_resource"
        neut.raw_resource = {"id": "42"}

        neut.delete()
        neut.user.neutron().delete_some_resource.assert_called_once_with("42")

    def test_list(self):
        neut = self.get_neutron_mixin()
        neut.user = mock.MagicMock()
        neut._resource = "some_resource"
        neut.tenant_uuid = "user_tenant"

        some_resources = [{"tenant_id": neut.tenant_uuid}, {"tenant_id": "a"}]
        neut.user.neutron().list_some_resources.return_value = {
            "some_resources": some_resources
        }

        self.assertEqual([some_resources[0]], list(neut.list()))

        neut.user.neutron().list_some_resources.assert_called_once_with(
            {"tenant_id": neut.tenant_uuid})


class NeutronPortTestCase(test.TestCase):

    def test_delete(self):
        raw_res = {"device_owner": "abbabaab", "id": "some_id"}
        user = mock.MagicMock()

        resources.NeutronPort(resource=raw_res, user=user).delete()

        user.neutron().delete_port.assert_called_once_with(raw_res["id"])

    def test_delete_port_raise_exception(self):
        raw_res = {"device_owner": "abbabaab", "id": "some_id"}
        user = mock.MagicMock()
        user.neutron().delete_port.side_effect = (
            neutron_exceptions.PortNotFoundClient)

        resources.NeutronPort(resource=raw_res, user=user).delete()

        user.neutron().delete_port.assert_called_once_with(raw_res["id"])

    def test_delete_port_device_owner(self):
        raw_res = {
            "device_owner": "network:router_interface",
            "id": "some_id",
            "device_id": "dev_id"
        }
        user = mock.MagicMock()

        resources.NeutronPort(resource=raw_res, user=user).delete()

        user.neutron().remove_interface_router.assert_called_once_with(
            raw_res["device_id"], {"port_id": raw_res["id"]})


class NeutronQuotaTestCase(test.TestCase):

    @mock.patch("%s.NeutronQuota._manager" % BASE)
    def test_delete(self, mock_manager):
        user = mock.MagicMock()
        resources.NeutronQuota(user=user, tenant_uuid="fake").delete()
        mock_manager().delete_quota.assert_called_once_with("fake")

    def test__manager(self):
        admin = mock.MagicMock(neutron=mock.Mock(return_value="foo"))
        res = resources.NeutronQuota(admin=admin, tenant_uuid="fake")
        res._manager()
        self.assertEqual("foo", getattr(admin, res._service)())


class GlanceImageTestCase(test.TestCase):

    @mock.patch("%s.GlanceImage._manager" % BASE)
    def test_list(self, mock_manager):
        glance = resources.GlanceImage()
        glance.tenant_uuid = mock.MagicMock()

        mock_manager().list.return_value = ["a", "b", "c"]

        self.assertEqual(["a", "b", "c"], glance.list())
        mock_manager().list.assert_called_once_with(owner=glance.tenant_uuid)


class CeilometerTestCase(test.TestCase):

    def test_id(self):
        ceil = resources.CeilometerAlarms()
        ceil.raw_resource = mock.MagicMock()
        self.assertEqual(ceil.raw_resource.alarm_id, ceil.id())

    @mock.patch("%s.CeilometerAlarms._manager" % BASE)
    def test_list(self, mock_manager):

        ceil = resources.CeilometerAlarms()
        ceil.tenant_uuid = mock.MagicMock()
        mock_manager().list.return_value = ["a", "b", "c"]
        mock_manager.reset_mock()

        self.assertEqual(["a", "b", "c"], ceil.list())
        mock_manager().list.assert_called_once_with(
            q=[{"field": "project_id", "op": "eq", "value": ceil.tenant_uuid}])


class ZaqarQueuesTestCase(test.TestCase):

    def test_list(self):
        user = mock.Mock()
        zaqar = resources.ZaqarQueues(user=user)
        zaqar.list()
        user.zaqar().queues.assert_called_once_with()


class KeystoneMixinTestCase(test.TestCase):

    def test_is_deleted(self):
        self.assertTrue(resources.KeystoneMixin().is_deleted())

    def get_keystone_mixin(self):
        kmixin = resources.KeystoneMixin()
        kmixin._service = "keystone"
        return kmixin

    @mock.patch("%s.keystone_wrapper.wrap" % BASE)
    def test_manager(self, mock_wrap):
        keystone_mixin = self.get_keystone_mixin()
        keystone_mixin.admin = mock.MagicMock()
        self.assertEqual(mock_wrap.return_value, keystone_mixin._manager())
        mock_wrap.assert_called_once_with(
            keystone_mixin.admin.keystone.return_value)

    @mock.patch("%s.keystone_wrapper.wrap" % BASE)
    def test_delete(self, mock_wrap):
        keystone_mixin = self.get_keystone_mixin()
        keystone_mixin._resource = "some_resource"
        keystone_mixin.id = lambda: "id_a"
        keystone_mixin.admin = mock.MagicMock()

        keystone_mixin.delete()
        mock_wrap.assert_called_once_with(
            keystone_mixin.admin.keystone.return_value)
        mock_wrap().delete_some_resource.assert_called_once_with("id_a")

    @mock.patch("%s.keystone_wrapper.wrap" % BASE)
    def test_list(self, mock_wrap):
        keystone_mixin = self.get_keystone_mixin()
        keystone_mixin._resource = "some_resource2"
        keystone_mixin.admin = mock.MagicMock()

        result = [mock.MagicMock(), mock.MagicMock(), mock.MagicMock()]
        prefix = keystone_utils.KeystoneScenario.RESOURCE_NAME_PREFIX
        result[0].name = prefix + "keystone-a"
        result[1].name = prefix + "keystone-b"
        result[2].name = "not_a_keystone_pattern"

        mock_wrap().list_some_resource2s.return_value = result

        self.assertSequenceEqual(result[:2], keystone_mixin.list())
        mock_wrap().list_some_resource2s.assert_called_once_with()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tests.hacking import checks
from tests.unit import test


class HackingTestCase(test.TestCase):

    def test__parse_assert_mock_str(self):
        pos, method, obj = checks._parse_assert_mock_str(
            "mock_clients.fake().quotas.delete.assert_called_once()")
        self.assertEqual("assert_called_once", method)
        self.assertEqual("mock_clients.fake().quotas.delete", obj)

    def test__parse_assert_mock_str_no_assert(self):
        pos, method, obj = checks._parse_assert_mock_str(
            "mock_clients.fake().quotas.delete.")
        self.assertIsNone(pos)
        self.assertIsNone(method)
        self.assertIsNone(obj)

    def test_correct_usage_of_assert_from_mock(self):
        correct_method_names = ["assert_any_call", "assert_called_once_with",
                                "assert_called_with", "assert_has_calls"]
        for name in correct_method_names:
            self.assertEqual(0, len(
                list(checks.check_assert_methods_from_mock(
                    "some_mock.%s(asd)" % name, "./tests/fake/test"))))

    def test_wrong_usage_of_broad_assert_from_mock(self):
        fake_method = "rtfm.assert_something()"

        actual_number, actual_msg = next(checks.check_assert_methods_from_mock(
            fake_method, "./tests/fake/test"))
        self.assertEqual(4, actual_number)
        self.assertTrue(actual_msg.startswith("N301"))

    def test_wrong_usage_of_assert_called_from_mock(self):
        fake_method = "rtfm.assert_called()"

        actual_number, actual_msg = next(checks.check_assert_methods_from_mock(
            fake_method, "./tests/fake/test"))
        self.assertEqual(4, actual_number)
        self.assertTrue(actual_msg.startswith("N302"))

    def test_wrong_usage_of_assert_called_once_from_mock(self):
        fake_method = "rtfm.assert_called_once()"

        actual_number, actual_msg = next(checks.check_assert_methods_from_mock(
            fake_method, "./tests/fake/test"))
        self.assertEqual(4, actual_number)
        self.assertTrue(actual_msg.startswith("N303"))

    def test_check_wrong_logging_import(self):
        bad_imports = ["from oslo_log import log",
                       "import oslo_log",
                       "import logging"]
        good_imports = ["from rally.common import log",
                        "from rally.common.log",
                        "import rally.common.log"]

        for bad_import in bad_imports:
            checkres = checks.check_import_of_logging(bad_import, "fakefile")
            self.assertIsNotNone(next(checkres))

        for bad_import in bad_imports:
            checkres = checks.check_import_of_logging(bad_import,
                                                      "./rally/common/log.py")
            self.assertEqual([], list(checkres))

        for good_import in good_imports:
            checkres = checks.check_import_of_logging(good_import,
                                                      "fakefile")
            self.assertEqual([], list(checkres))

    def test_no_translate_debug_logs(self):
        self.assertEqual(len(list(checks.no_translate_debug_logs(
            "LOG.debug(_('foo'))"))), 1)

        self.assertEqual(len(list(checks.no_translate_debug_logs(
            "LOG.debug('foo')"))), 0)

        self.assertEqual(len(list(checks.no_translate_debug_logs(
            "LOG.info(_('foo'))"))), 0)

    def test_no_use_conf_debug_check(self):
        self.assertEqual(len(list(checks.no_use_conf_debug_check(
            "if CONF.debug:", "fakefile"))), 1)

        self.assertEqual(len(list(checks.no_use_conf_debug_check(
            "if cfg.CONF.debug", "fakefile"))), 1)

        self.assertEqual(len(list(checks.no_use_conf_debug_check(
            "if logging.is_debug()", "fakefile"))), 0)

    def test_assert_true_instance(self):
        self.assertEqual(len(list(checks.assert_true_instance(
            "self.assertTrue(isinstance(e, "
            "exception.BuildAbortException))"))), 1)

        self.assertEqual(
            len(list(checks.assert_true_instance("self.assertTrue()"))), 0)

    def test_assert_equal_type(self):
        self.assertEqual(len(list(checks.assert_equal_type(
            "self.assertEqual(type(als['QuicAssist']), list)"))), 1)

        self.assertEqual(
            len(list(checks.assert_equal_type("self.assertTrue()"))), 0)

    def test_check_iteritems_method(self):
        self.assertEqual(len(list(checks.check_iteritems_method(
            "dict.iteritems()"))), 1)

        self.assertEqual(len(list(checks.check_iteritems_method(
            "iteritems(dict)"))), 0)

        self.assertEqual(len(list(checks.check_iteritems_method(
            "dict.items()"))), 0)

    def test_check_concatenate_dict_with_plus_operand(self):
        self.assertEqual(len(list(
            checks.check_concatenate_dict_with_plus_operand(
                "dict1.items() + dict2.items()"))), 1)

        self.assertEqual(len(list(
            checks.check_concatenate_dict_with_plus_operand(
                "dict(self.endpoint.to_dict().items() + "
                "endpoint.items())"))), 1)

        self.assertEqual(len(list(
            checks.check_concatenate_dict_with_plus_operand(
                "dict(self.endpoint.to_dict().items() + "
                "endpoint.items() + something.items())"))), 1)

        self.assertEqual(len(list(
            checks.check_concatenate_dict_with_plus_operand(
                "dict1.item() + dict2.item()"))), 0)

        self.assertEqual(len(list(
            checks.check_concatenate_dict_with_plus_operand(
                "obj.getitems() + obj.getitems()"))), 0)

        self.assertEqual(len(list(
            checks.check_concatenate_dict_with_plus_operand(
                "obj.getitems() + dict2.items()"))), 0)

    def test_check_basestring_method(self):
        self.assertEqual(len(list(checks.check_basestring_method(
            "basestring"))), 1)

        self.assertEqual(len(list(checks.check_basestring_method(
            "six.string_types"))), 0)

    def test_check_StringIO_method(self):
        self.assertEqual(len(list(checks.check_StringIO_method(
            "StringIO.StringIO()"))), 1)

        self.assertEqual(len(list(checks.check_StringIO_method(
            "six.moves.StringIO()"))), 0)

    def test_check_urlparse_method(self):
        self.assertEqual(len(list(checks.check_urlparse_method(
            "urlparse.urlparse(url)"))), 1)

        self.assertEqual(len(list(checks.check_urlparse_method(
            "six.moves.urllib.parse.urlparse(url)"))), 0)

    def test_check_itertools_imap_method(self):
        self.assertEqual(len(list(checks.check_itertools_imap_method(
            "itertools.imap()"))), 1)

        self.assertEqual(len(list(checks.check_itertools_imap_method(
            "six.moves.map()"))), 0)

    def test_check_xrange_imap_method(self):
        self.assertEqual(len(list(checks.check_xrange_method(
            "xrange()"))), 1)

        self.assertEqual(len(list(checks.check_xrange_method(
            "six.moves.range()"))), 0)

    def test_check_string_lower_upper_case_method(self):
        self.assertEqual(len(list(checks.check_string_lower_upper_case_method(
            "string.lowercase[:16]"))), 1)

        self.assertEqual(len(list(checks.check_string_lower_upper_case_method(
            "string.uppercase[:16]"))), 1)

        self.assertEqual(len(list(checks.check_string_lower_upper_case_method(
            "string.ascii_lowercase[:16]"))), 0)

        self.assertEqual(len(list(checks.check_string_lower_upper_case_method(
            "string.ascii_uppercase[:16]"))), 0)

    def test_check_next_on_iterator_method(self):
        self.assertEqual(len(list(checks.check_next_on_iterator_method(
            "iterator.next()"))), 1)

        self.assertEqual(len(list(checks.check_next_on_iterator_method(
            "next(iterator)"))), 0)

    def test_assert_equal_none(self):
        self.assertEqual(len(list(checks.assert_equal_none(
            "self.assertEqual(A, None)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_none(
            "self.assertEqual(None, A)"))), 1)

        self.assertEqual(
            len(list(checks.assert_equal_none("self.assertIsNone()"))), 0)

    def test_assert_true_or_false_with_in_or_not_in(self):
        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A in B)"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertFalse(A in B)"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A not in B)"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertFalse(A not in B)"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A in B, 'some message')"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertFalse(A in B, 'some message')"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A not in B, 'some message')"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertFalse(A not in B, 'some message')"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A in 'some string with spaces')"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A in 'some string with spaces')"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A in ['1', '2', '3'])"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(A in [1, 2, 3])"))), 1)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(any(A > 5 for A in B))"))), 0)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertTrue(any(A > 5 for A in B), 'some message')"))), 0)

        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
            "self.assertFalse(some in list1 and some2 in list2)"))), 0)

    def test_assert_equal_in(self):
        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(a in b, True)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(a not in b, True)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual('str' in 'string', True)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual('str' not in 'string', True)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(any(a==1 for a in b), True)"))), 0)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(True, a in b)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(True, a not in b)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(True, 'str' in 'string')"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(True, 'str' not in 'string')"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(True, any(a==1 for a in b))"))), 0)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(a in b, False)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(a not in b, False)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual('str' in 'string', False)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual('str' not in 'string', False)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(any(a==1 for a in b), False)"))), 0)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(False, a in b)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(False, a not in b)"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(False, 'str' in 'string')"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(False, 'str' not in 'string')"))), 1)

        self.assertEqual(len(list(checks.assert_equal_in(
            "self.assertEqual(False, any(a==1 for a in b))"))), 0)

    def test_check_no_direct_rally_objects_import(self):

        bad_imports = ["from rally.objects import task",
                       "import rally.objects.task"]

        good_import = "from rally import objects"

        for bad_import in bad_imports:
            checkres = checks.check_no_direct_rally_objects_import(bad_import,
                                                                   "fakefile")
            self.assertIsNotNone(next(checkres))

        for bad_import in bad_imports:
            checkres = checks.check_no_direct_rally_objects_import(
                bad_import, "./rally/objects/__init__.py")
            self.assertEqual([], list(checkres))

        checkres = checks.check_no_direct_rally_objects_import(good_import,
                                                               "fakefile")
        self.assertEqual([], list(checkres))

    def test_check_no_oslo_deprecated_import(self):

        bad_imports = ["from oslo.config",
                       "import oslo.config,"
                       "from oslo.db",
                       "import oslo.db,"
                       "from oslo.i18n",
                       "import oslo.i18n,"
                       "from oslo.serialization",
                       "import oslo.serialization,"
                       "from oslo.utils",
                       "import oslo.utils,"]

        for bad_import in bad_imports:
            checkres = checks.check_no_oslo_deprecated_import(bad_import,
                                                              "fakefile")
            self.assertIsNotNone(next(checkres))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import mock

from rally.benchmark.scenarios.dummy import dummy
from tests.unit import test


class DummyTestCase(test.TestCase):

    @mock.patch("rally.benchmark.scenarios.dummy.dummy.time")
    def test_dummy(self, mock_sleep):
        scenario = dummy.Dummy()
        scenario.sleep_between = mock.MagicMock()
        scenario.dummy()
        self.assertFalse(mock_sleep.sleep.called)

        scenario.dummy(sleep=10)
        mock_sleep.sleep.assert_called_once_with(10)

    @mock.patch("rally.benchmark.scenarios.dummy.dummy.time")
    def test_dummy_exception(self, mock_sleep):
        scenario = dummy.Dummy()

        size_of_message = 5
        self.assertRaises(dummy.DummyScenarioException,
                          scenario.dummy_exception, size_of_message, sleep=10)
        mock_sleep.sleep.assert_called_once_with(10)

    def test_dummy_exception_probability(self):
        scenario = dummy.Dummy()

        # should not raise an exception as probability is 0
        for i in range(100):
            scenario.dummy_exception_probability(exception_probability=0)

        # should always raise an exception as probability is 1
        for i in range(100):
            self.assertRaises(dummy.DummyScenarioException,
                              scenario.dummy_exception_probability,
                              exception_probability=1)

    def test_dummy_dummy_with_scenario_output(self):
        scenario = dummy.Dummy()
        result = scenario.dummy_with_scenario_output()
        self.assertEqual(result["errors"], "")
        # Since the data is generated in random,
        # checking for not None
        self.assertNotEqual(result["data"], None)

    def test_dummy_random_fail_in_atomic(self):
        scenario = dummy.Dummy()

        for i in range(10):
            scenario.dummy_random_fail_in_atomic(exception_probability=0)

        for i in range(10):
            self.assertRaises(KeyError,
                              scenario.dummy_random_fail_in_atomic,
                              exception_probability=1)



# Copyright 2014: Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections
import multiprocessing
import random
import threading
import time

from rally.benchmark.runners import base
from rally.common import log as logging
from rally.common import utils
from rally import consts


LOG = logging.getLogger(__name__)


def _worker_process(queue, iteration_gen, timeout, rps, times,
                    context, cls, method_name, args, aborted):
    """Start scenario within threads.

    Spawn N threads per second. Each thread runs the scenario once, and appends
    the result to the queue.

    :param queue: queue object to append results
    :param iteration_gen: next iteration number generator
    :param timeout: operation's timeout
    :param rps: number of scenario iterations to be run per one second
    :param times: total number of scenario iterations to be run
    :param context: scenario context object
    :param cls: scenario class
    :param method_name: scenario method name
    :param args: scenario args
    :param aborted: multiprocessing.Event that aborts load generation if
                    the flag is set
    """

    pool = collections.deque()
    start = time.time()
    sleep = 1.0 / rps

    base._log_worker_info(times=times, rps=rps, timeout=timeout,
                          cls=cls, method_name=method_name, args=args)

    # Injecting timeout to exclude situations, where start time and
    # actual time are neglible close

    randsleep_delay = random.randint(int(sleep / 2 * 100), int(sleep * 100))
    time.sleep(randsleep_delay / 100.0)

    i = 0
    while i < times and not aborted.is_set():
        scenario_context = base._get_scenario_context(context)
        scenario_args = (next(iteration_gen), cls, method_name,
                         scenario_context, args)
        worker_args = (queue, scenario_args)
        thread = threading.Thread(target=base._worker_thread,
                                  args=worker_args)
        i += 1
        thread.start()
        pool.append(thread)

        time_gap = time.time() - start
        real_rps = i / time_gap if time_gap else "Infinity"

        LOG.debug("Worker: %s rps: %s (requested rps: %s)" %
                  (i, real_rps, rps))

        # try to join latest thread(s) until it finished, or until time to
        # start new thread
        while i / (time.time() - start) > rps:
            if pool:
                pool[0].join(sleep)
                if not pool[0].isAlive():
                    pool.popleft()
            else:
                time.sleep(sleep)

    while pool:
        thr = pool.popleft()
        thr.join()


class RPSScenarioRunner(base.ScenarioRunner):
    """Scenario runner that does the job with specified frequency.

    Every single benchmark scenario iteration is executed with specified
    frequency (runs per second) in a pool of processes. The scenario will be
    launched for a fixed number of times in total (specified in the config).

    An example of a rps scenario is booting 1 VM onse per second. This
    execution type is thus very helpful in understanding the maximal load that
    a certain cloud can handle.
    """

    __execution_type__ = consts.RunnerType.RPS

    CONFIG_SCHEMA = {
        "type": "object",
        "$schema": consts.JSON_SCHEMA,
        "properties": {
            "type": {
                "type": "string"
            },
            "times": {
                "type": "integer",
                "minimum": 1
            },
            "rps": {
                "type": "number",
                "minimum": 1
            },
            "timeout": {
                "type": "number",
            },
        },
        "additionalProperties": False
    }

    def _run_scenario(self, cls, method_name, context, args):
        """Runs the specified benchmark scenario with given arguments.

        Every single benchmark scenario iteration is executed with specified
        frequency (runs per second) in a pool of processes. The scenario will
        be launched for a fixed number of times in total (specified in the
        config).

        :param cls: The Scenario class where the scenario is implemented
        :param method_name: Name of the method that implements the scenario
        :param context: Benchmark context that contains users, admin & other
                        information, that was created before benchmark started.
        :param args: Arguments to call the scenario method with

        :returns: List of results fore each single scenario iteration,
                  where each result is a dictionary
        """
        times = self.config["times"]
        timeout = self.config.get("timeout", 0)  # 0 means no timeout
        iteration_gen = utils.RAMInt()
        cpu_count = multiprocessing.cpu_count()
        processes_to_start = min(cpu_count, times)
        rps_per_worker = float(self.config["rps"]) / processes_to_start
        times_per_worker, times_overhead = divmod(times, processes_to_start)

        self._log_debug_info(times=times, timeout=timeout, cpu_count=cpu_count,
                             processes_to_start=processes_to_start,
                             rps_per_worker=rps_per_worker,
                             times_per_worker=times_per_worker,
                             times_overhead=times_overhead)

        result_queue = multiprocessing.Queue()

        def worker_args_gen(times_overhead):
            while True:
                yield (result_queue, iteration_gen, timeout, rps_per_worker,
                       times_per_worker + (times_overhead and 1),
                       context, cls, method_name, args, self.aborted)
                if times_overhead:
                    times_overhead -= 1

        process_pool = self._create_process_pool(
            processes_to_start, _worker_process,
            worker_args_gen(times_overhead))
        self._join_processes(process_pool, result_queue)
